@@ -22,6 +22,10 @@
 #define COBJMACROS
 #define CONST_VTABLE
 
+#include <windef.h>
+#include <winbase.h>
+#include <winuser.h>
+
 #include <atlbase.h>
 #include <mshtml.h>
 
@@ -562,6 +566,32 @@ static void test_source_iface(void)
     ok(maj_ver == 4 && min_ver == 0, "ver = %d.%d\n", maj_ver, min_ver);
 }
 
+static void test_ax_win(void)
+{
+    BOOL ret;
+    WNDCLASSEXW wcex;
+    static const WCHAR AtlAxWin100[] = {'A','t','l','A','x','W','i','n','1','0','0',0};
+    static const WCHAR AtlAxWinLic100[] = {'A','t','l','A','x','W','i','n','L','i','c','1','0','0',0};
+    static HMODULE hinstance = 0;
+
+    ret = AtlAxWinInit();
+    ok(ret, "AtlAxWinInit failed\n");
+
+    hinstance = GetModuleHandleA(NULL);
+
+    memset(&wcex, 0, sizeof(wcex));
+    wcex.cbSize = sizeof(wcex);
+    ret = GetClassInfoExW(hinstance, AtlAxWin100, &wcex);
+    ok(ret, "AtlAxWin100 has not registered\n");
+    ok(wcex.style == (CS_GLOBALCLASS | CS_DBLCLKS), "wcex.style %08x\n", wcex.style);
+
+    memset(&wcex, 0, sizeof(wcex));
+    wcex.cbSize = sizeof(wcex);
+    ret = GetClassInfoExW(hinstance, AtlAxWinLic100, &wcex);
+    ok(ret, "AtlAxWinLic100 has not registered\n");
+    ok(wcex.style == (CS_GLOBALCLASS | CS_DBLCLKS), "wcex.style %08x\n", wcex.style);
+}
+
 START_TEST(atl)
 {
     CoInitialize(NULL);
@@ -571,6 +601,7 @@ START_TEST(atl)
     test_typelib();
     test_cp();
     test_source_iface();
+    test_ax_win();
 
     CoUninitialize();
 }
