@@ -19,7 +19,11 @@
  *
  */
 
+#define COBJMACROS
+#define CONST_VTABLE
+
 #include <stdarg.h>
+#include <stdio.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -35,8 +39,9 @@
 
 /* ########################### */
 
+extern const IID IID_IObjectWithSite;
+
 static HMODULE  hcomdlg32;
-static HRESULT (WINAPI * pPrintDlgExA)(LPPRINTDLGEXA);
 static HRESULT (WINAPI * pPrintDlgExW)(LPPRINTDLGEXW);
 
 /* ########################### */
@@ -44,25 +49,16 @@ static HRESULT (WINAPI * pPrintDlgExW)(LPPRINTDLGEXW);
 static const CHAR emptyA[] = "";
 static const CHAR PrinterPortsA[] = "PrinterPorts";
 
-/* ########################### */
-
-static LPCSTR load_functions(void)
+static const char *debugstr_guid(const GUID *guid)
 {
-    LPCSTR  ptr;
-
-    ptr = "comdlg32.dll";
-    hcomdlg32 = GetModuleHandleA(ptr);
-
-    ptr = "PrintDlgExA";
-    pPrintDlgExA = (void *) GetProcAddress(hcomdlg32, ptr);
-    if (!pPrintDlgExA) return ptr;
-
-    ptr = "PrintDlgExW";
-    pPrintDlgExW = (void *) GetProcAddress(hcomdlg32, ptr);
-    if (!pPrintDlgExW) return ptr;
-
-    return NULL;
+    static char buf[50];
 
+    if (!guid) return "(null)";
+    sprintf(buf, "{%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x}",
+            guid->Data1, guid->Data2, guid->Data3, guid->Data4[0],
+            guid->Data4[1], guid->Data4[2], guid->Data4[3], guid->Data4[4],
+            guid->Data4[5], guid->Data4[6], guid->Data4[7]);
+    return buf;
 }
 
 /* ########################### */
@@ -200,9 +196,7 @@ static void test_PrintDlgA(void)
 
         /* The Driver Entry does not include a Path */
         ptr = strrchr(driver, '\\');
-        todo_wine {
         ok( ptr == NULL, "got %p for '%s' (expected NULL for a simple name)\n", ptr, driver);
-        }
 
         /* The Driver Entry does not have an extension (fixed to ".drv") */
         ptr = strrchr(driver, '.');
@@ -220,11 +214,8 @@ static void test_PrintDlgA(void)
             res, GetLastError(), ptr, buffer);
 
         if (ptr) ptr[0] = '\0';
-        todo_wine {
         ok( lstrcmpiA(driver, buffer) == 0,
             "got driver '%s' (expected '%s')\n", driver, buffer);
-        }
-
     }
 
     GlobalUnlock(pDlg->hDevNames);
@@ -237,20 +228,115 @@ static void test_PrintDlgA(void)
 
 /* ########################### */
 
+static HRESULT WINAPI callback_QueryInterface(IPrintDialogCallback *iface,
+                                              REFIID riid, void **ppv)
+{
+    ok(0, "callback_QueryInterface(%s): unexpected call\n", debugstr_guid(riid));
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI callback_AddRef(IPrintDialogCallback *iface)
+{
+    trace("callback_AddRef\n");
+    return 2;
+}
+
+static ULONG WINAPI callback_Release(IPrintDialogCallback *iface)
+{
+    trace("callback_Release\n");
+    return 1;
+}
+
+static HRESULT WINAPI callback_InitDone(IPrintDialogCallback *iface)
+{
+    trace("callback_InitDone\n");
+    return S_OK;
+}
+
+static HRESULT WINAPI callback_SelectionChange(IPrintDialogCallback *iface)
+{
+    trace("callback_SelectionChange\n");
+    return S_OK;
+}
+
+static HRESULT WINAPI callback_HandleMessage(IPrintDialogCallback *iface,
+    HWND hdlg, UINT msg, WPARAM wp, LPARAM lp, LRESULT *res)
+{
+    trace("callback_HandleMessage %p,%04x,%lx,%lx,%p\n", hdlg, msg, wp, lp, res);
+    /* *res = PD_RESULT_PRINT; */
+    return S_OK;
+}
+
+static const IPrintDialogCallbackVtbl callback_Vtbl =
+{
+    callback_QueryInterface,
+    callback_AddRef,
+    callback_Release,
+    callback_InitDone,
+    callback_SelectionChange,
+    callback_HandleMessage
+};
+
+static IPrintDialogCallback callback = { &callback_Vtbl };
+
+static HRESULT WINAPI unknown_QueryInterface(IUnknown *iface, REFIID riid, void **ppv)
+{
+    trace("unknown_QueryInterface %s\n", debugstr_guid(riid));
+
+    if (IsEqualGUID(riid, &IID_IPrintDialogCallback))
+    {
+        *ppv = &callback;
+        return S_OK;
+    }
+    else if (IsEqualGUID(riid, &IID_IObjectWithSite))
+    {
+        *ppv = NULL;
+        return E_NOINTERFACE;
+    }
+
+    ok(0, "unexpected IID %s\n", debugstr_guid(riid));
+    *ppv = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI unknown_AddRef(IUnknown *iface)
+{
+    trace("unknown_AddRef\n");
+    return 2;
+}
+
+static ULONG WINAPI unknown_Release(IUnknown *iface)
+{
+    trace("unknown_Release\n");
+    return 1;
+}
+
+static const IUnknownVtbl unknown_Vtbl =
+{
+    unknown_QueryInterface,
+    unknown_AddRef,
+    unknown_Release
+};
+
+static IUnknown unknown = { &unknown_Vtbl };
+
 static void test_PrintDlgExW(void)
 {
+    PRINTPAGERANGE pagerange[2];
     LPPRINTDLGEXW pDlg;
+    DEVNAMES *dn;
     HRESULT res;
 
+    /* PrintDlgEx not present before w2k */
+    if (!pPrintDlgExW) {
+        skip("PrintDlgExW not available\n");
+        return;
+    }
+
     /* Set CommDlgExtendedError != 0 */
     PrintDlg(NULL);
     SetLastError(0xdeadbeef);
     res = pPrintDlgExW(NULL);
-    if(res == E_NOTIMPL)
-    {
-        win_skip("PrintDlgExW returns not implemented\n");
-        return;
-    }
     ok( (res == E_INVALIDARG),
         "got 0x%x with %u and %u (expected 'E_INVALIDARG')\n",
         res, GetLastError(), CommDlgExtendedError());
@@ -288,10 +374,136 @@ static void test_PrintDlgExW(void)
         "got 0x%x with %u and %u (expected 'E_HANDLE')\n",
         res, GetLastError(), CommDlgExtendedError());
 
+    /* nStartPage must be START_PAGE_GENERAL for the general page or a valid property sheet index */
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_RETURNDEFAULT | PD_NOWARNING | PD_NOPAGENUMS;
+    res = pPrintDlgExW(pDlg);
+    ok((res == E_INVALIDARG), "got 0x%x (expected 'E_INVALIDARG')\n", res);
+
+    /* Use PD_NOPAGENUMS or set nMaxPageRanges and lpPageRanges */
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_RETURNDEFAULT | PD_NOWARNING;
+    pDlg->nStartPage = START_PAGE_GENERAL;
+    res = pPrintDlgExW(pDlg);
+    ok((res == E_INVALIDARG), "got 0x%x (expected 'E_INVALIDARG')\n", res);
+
+    /* this is invalid: a valid lpPageRanges with 0 for nMaxPageRanges */
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_RETURNDEFAULT | PD_NOWARNING;
+    pDlg->lpPageRanges = pagerange;
+    pDlg->nStartPage = START_PAGE_GENERAL;
+    res = pPrintDlgExW(pDlg);
+    ok((res == E_INVALIDARG), "got 0x%x (expected 'E_INVALIDARG')\n", res);
+
+    /* this is invalid: NULL for lpPageRanges with a valid nMaxPageRanges */
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_RETURNDEFAULT | PD_NOWARNING;
+    pDlg->nMaxPageRanges = 1;
+    pDlg->nStartPage = START_PAGE_GENERAL;
+    res = pPrintDlgExW(pDlg);
+    ok((res == E_INVALIDARG), "got 0x%x (expected 'E_INVALIDARG')\n", res);
+
+    /* this works: lpPageRanges with a valid nMaxPageRanges */
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_RETURNDEFAULT | PD_NOWARNING;
+    pDlg->nMaxPageRanges = 1;
+    pDlg->lpPageRanges = pagerange;
+    pDlg->nStartPage = START_PAGE_GENERAL;
+    res = pPrintDlgExW(pDlg);
+    if (res == E_FAIL)
+    {
+        skip("No printer configured.\n");
+        HeapFree(GetProcessHeap(), 0, pDlg);
+        return;
+    }
+
+    ok(res == S_OK, "got 0x%x (expected S_OK)\n", res);
+
+    dn = GlobalLock(pDlg->hDevNames);
+    ok(dn != NULL, "expected '!= NULL' for GlobalLock(%p)\n",pDlg->hDevNames);
+    if (dn)
+    {
+        ok(dn->wDriverOffset, "(expected '!= 0' for wDriverOffset)\n");
+        ok(dn->wDeviceOffset, "(expected '!= 0' for wDeviceOffset)\n");
+        ok(dn->wOutputOffset, "(expected '!= 0' for wOutputOffset)\n");
+        ok(dn->wDefault == DN_DEFAULTPRN, "got 0x%x (expected DN_DEFAULTPRN)\n", dn->wDefault);
+
+        GlobalUnlock(pDlg->hDevNames);
+    }
+    GlobalFree(pDlg->hDevMode);
+    GlobalFree(pDlg->hDevNames);
+
+    /* this works also: PD_NOPAGENUMS */
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_RETURNDEFAULT | PD_NOWARNING | PD_NOPAGENUMS;
+    pDlg->nStartPage = START_PAGE_GENERAL;
+    res = pPrintDlgExW(pDlg);
+    ok(res == S_OK, "got 0x%x (expected S_OK)\n", res);
+    GlobalFree(pDlg->hDevMode);
+    GlobalFree(pDlg->hDevNames);
+
+    /* this works: PD_RETURNDC with PD_RETURNDEFAULT */
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_RETURNDEFAULT | PD_NOWARNING | PD_NOPAGENUMS | PD_RETURNDC;
+    pDlg->nStartPage = START_PAGE_GENERAL;
+    res = pPrintDlgExW(pDlg);
+    ok(res == S_OK, "got 0x%x (expected S_OK)\n", res);
+    ok(pDlg->hDC != NULL, "HDC missing for PD_RETURNDC\n");
+    GlobalFree(pDlg->hDevMode);
+    GlobalFree(pDlg->hDevNames);
+    DeleteDC(pDlg->hDC);
+
+    /* this works: PD_RETURNIC with PD_RETURNDEFAULT */
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_RETURNDEFAULT | PD_NOWARNING | PD_NOPAGENUMS | PD_RETURNIC;
+    pDlg->nStartPage = START_PAGE_GENERAL;
+    res = pPrintDlgExW(pDlg);
+    ok(res == S_OK, "got 0x%x (expected S_OK)\n", res);
+    ok(pDlg->hDC != NULL, "HDC missing for PD_RETURNIC\n");
+    GlobalFree(pDlg->hDevMode);
+    GlobalFree(pDlg->hDevNames);
+    DeleteDC(pDlg->hDC);
+
+    /* interactive PrintDlgEx tests */
+
+    if (!winetest_interactive)
+    {
+        skip("interactive PrintDlgEx tests (set WINETEST_INTERACTIVE=1)\n");
+        return;
+    }
+
+    ZeroMemory(pDlg, sizeof(PRINTDLGEXW));
+    pDlg->lStructSize = sizeof(PRINTDLGEXW);
+    pDlg->hwndOwner = GetDesktopWindow();
+    pDlg->Flags = PD_NOPAGENUMS | PD_RETURNIC;
+    pDlg->nStartPage = START_PAGE_GENERAL;
+    pDlg->lpCallback = &unknown;
+    pDlg->dwResultAction = S_OK;
+    res = pPrintDlgExW(pDlg);
+    ok(res == S_OK, "got 0x%x (expected S_OK)\n", res);
+    ok(pDlg->dwResultAction == PD_RESULT_PRINT, "expected PD_RESULT_PRINT, got %#x\n", pDlg->dwResultAction);
+    ok(pDlg->hDC != NULL, "HDC missing for PD_RETURNIC\n");
+    GlobalFree(pDlg->hDevMode);
+    GlobalFree(pDlg->hDevNames);
+    DeleteDC(pDlg->hDC);
 
     HeapFree(GetProcessHeap(), 0, pDlg);
-    return;
-
 }
 
 static BOOL abort_proc_called = FALSE;
@@ -320,18 +532,14 @@ static void test_abort_proc(void)
     if (!PrintDlgA(&pd))
     {
         skip("No default printer available.\n");
-        ok(DeleteFileA(filename), "Failed to delete temporary file\n");
-        return;
+        goto end;
     }
     GlobalFree(pd.hDevMode);
     GlobalFree(pd.hDevNames);
 
     ok(pd.hDC != NULL, "PrintDlg didn't return a DC.\n");
     if (!(print_dc = pd.hDC))
-    {
-        ok(DeleteFileA(filename), "Failed to delete temporary file\n");
-        return;
-    }
+        goto end;
 
     ok(SetAbortProc(print_dc, abort_proc) > 0, "SetAbortProc failed\n");
     ok(!abort_proc_called, "AbortProc got called unexpectedly by SetAbortProc.\n");
@@ -353,50 +561,38 @@ static void test_abort_proc(void)
         goto end;
     }
 
-    ok(abort_proc_called, "AbortProc didn't get called by StartDoc.\n");
-    abort_proc_called = FALSE;
+    /* StartDoc may or may not call abort proc */
 
+    abort_proc_called = FALSE;
     ok(StartPage(print_dc) > 0, "StartPage failed\n");
     ok(!abort_proc_called, "AbortProc got called unexpectedly by StartPage.\n");
     abort_proc_called = FALSE;
 
+    /* following functions sometimes call abort proc too */
     ok(FillRect(print_dc, &rect, (HBRUSH)(COLOR_BACKGROUND + 1)), "FillRect failed\n");
-    ok(!abort_proc_called, "AbortProc got called unexpectedly by StretchBlt.\n");
-    abort_proc_called = FALSE;
-
     ok(EndPage(print_dc) > 0, "EndPage failed\n");
-    ok(!abort_proc_called, "AbortProc got called unexpectedly by EndPage.\n");
-    abort_proc_called = FALSE;
-
     ok(EndDoc(print_dc) > 0, "EndDoc failed\n");
-    ok(!abort_proc_called, "AbortProc got called unexpectedly by EndDoc.\n");
-    abort_proc_called = FALSE;
 
+    abort_proc_called = FALSE;
     ok(DeleteDC(print_dc), "DeleteDC failed\n");
     ok(!abort_proc_called, "AbortProc got called unexpectedly by DeleteDC.\n");
     abort_proc_called = FALSE;
 
 end:
-    ok(DeleteFileA(filename), "Failed to delete temporary file\n");
+    SetLastError(0xdeadbeef);
+    if(!DeleteFileA(filename))
+        trace("Failed to delete temporary file (err = %x)\n", GetLastError());
 }
 
 /* ########################### */
 
 START_TEST(printdlg)
 {
-    LPCSTR  ptr;
-
-    ptr = load_functions();
+    hcomdlg32 = GetModuleHandleA("comdlg32.dll");
+    pPrintDlgExW = (void *) GetProcAddress(hcomdlg32, "PrintDlgExW");
 
     test_PageSetupDlgA();
     test_PrintDlgA();
-    test_abort_proc();
-
-    /* PrintDlgEx not present before w2k */
-    if (ptr) {
-        win_skip("%s\n", ptr);
-        return;
-    }
-
     test_PrintDlgExW();
+    test_abort_proc();
 }
