@@ -19,47 +19,46 @@
  */
 
 #include "config.h"
-#include <assert.h>
 #include "d3d9_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d9);
 
-static inline IDirect3DVertexBuffer9Impl *impl_from_IDirect3DVertexBuffer9(IDirect3DVertexBuffer9 *iface)
+static inline struct d3d9_vertexbuffer *impl_from_IDirect3DVertexBuffer9(IDirect3DVertexBuffer9 *iface)
 {
-    return CONTAINING_RECORD(iface, IDirect3DVertexBuffer9Impl, IDirect3DVertexBuffer9_iface);
+    return CONTAINING_RECORD(iface, struct d3d9_vertexbuffer, IDirect3DVertexBuffer9_iface);
 }
 
-static HRESULT WINAPI d3d9_vertexbuffer_QueryInterface(IDirect3DVertexBuffer9 *iface, REFIID riid, void **object)
+static HRESULT WINAPI d3d9_vertexbuffer_QueryInterface(IDirect3DVertexBuffer9 *iface, REFIID riid, void **out)
 {
-    TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), object);
+    TRACE("iface %p, riid %s, out %p.\n", iface, debugstr_guid(riid), out);
 
     if (IsEqualGUID(riid, &IID_IDirect3DVertexBuffer9)
             || IsEqualGUID(riid, &IID_IDirect3DResource9)
             || IsEqualGUID(riid, &IID_IUnknown))
     {
         IDirect3DVertexBuffer9_AddRef(iface);
-        *object = iface;
+        *out = iface;
         return S_OK;
     }
 
     WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(riid));
 
-    *object = NULL;
+    *out = NULL;
     return E_NOINTERFACE;
 }
 
 static ULONG WINAPI d3d9_vertexbuffer_AddRef(IDirect3DVertexBuffer9 *iface)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
-    ULONG refcount = InterlockedIncrement(&buffer->ref);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    ULONG refcount = InterlockedIncrement(&buffer->refcount);
 
     TRACE("%p increasing refcount to %u.\n", iface, refcount);
 
     if (refcount == 1)
     {
-        IDirect3DDevice9Ex_AddRef(buffer->parentDevice);
+        IDirect3DDevice9Ex_AddRef(buffer->parent_device);
         wined3d_mutex_lock();
-        wined3d_buffer_incref(buffer->wineD3DVertexBuffer);
+        wined3d_buffer_incref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
     }
 
@@ -68,17 +67,17 @@ static ULONG WINAPI d3d9_vertexbuffer_AddRef(IDirect3DVertexBuffer9 *iface)
 
 static ULONG WINAPI d3d9_vertexbuffer_Release(IDirect3DVertexBuffer9 *iface)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
-    ULONG refcount = InterlockedDecrement(&buffer->ref);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    ULONG refcount = InterlockedDecrement(&buffer->refcount);
 
     TRACE("%p decreasing refcount to %u.\n", iface, refcount);
 
     if (!refcount)
     {
-        IDirect3DDevice9Ex *device = buffer->parentDevice;
+        IDirect3DDevice9Ex *device = buffer->parent_device;
 
         wined3d_mutex_lock();
-        wined3d_buffer_decref(buffer->wineD3DVertexBuffer);
+        wined3d_buffer_decref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
 
         /* Release the device last, as it may cause the device to be destroyed. */
@@ -88,14 +87,13 @@ static ULONG WINAPI d3d9_vertexbuffer_Release(IDirect3DVertexBuffer9 *iface)
     return refcount;
 }
 
-static HRESULT WINAPI d3d9_vertexbuffer_GetDevice(IDirect3DVertexBuffer9 *iface,
-        IDirect3DDevice9 **device)
+static HRESULT WINAPI d3d9_vertexbuffer_GetDevice(IDirect3DVertexBuffer9 *iface, IDirect3DDevice9 **device)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = (IDirect3DDevice9 *)buffer->parentDevice;
+    *device = (IDirect3DDevice9 *)buffer->parent_device;
     IDirect3DDevice9_AddRef(*device);
 
     TRACE("Returning device %p.\n", *device);
@@ -106,7 +104,7 @@ static HRESULT WINAPI d3d9_vertexbuffer_GetDevice(IDirect3DVertexBuffer9 *iface,
 static HRESULT WINAPI d3d9_vertexbuffer_SetPrivateData(IDirect3DVertexBuffer9 *iface,
         REFGUID guid, const void *data, DWORD data_size, DWORD flags)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
@@ -114,7 +112,7 @@ static HRESULT WINAPI d3d9_vertexbuffer_SetPrivateData(IDirect3DVertexBuffer9 *i
             iface, debugstr_guid(guid), data, data_size, flags);
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DVertexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_set_private_data(resource, guid, data, data_size, flags);
     wined3d_mutex_unlock();
 
@@ -124,7 +122,7 @@ static HRESULT WINAPI d3d9_vertexbuffer_SetPrivateData(IDirect3DVertexBuffer9 *i
 static HRESULT WINAPI d3d9_vertexbuffer_GetPrivateData(IDirect3DVertexBuffer9 *iface,
         REFGUID guid, void *data, DWORD *data_size)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
@@ -132,7 +130,7 @@ static HRESULT WINAPI d3d9_vertexbuffer_GetPrivateData(IDirect3DVertexBuffer9 *i
             iface, debugstr_guid(guid), data, data_size);
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DVertexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_get_private_data(resource, guid, data, data_size);
     wined3d_mutex_unlock();
 
@@ -141,14 +139,14 @@ static HRESULT WINAPI d3d9_vertexbuffer_GetPrivateData(IDirect3DVertexBuffer9 *i
 
 static HRESULT WINAPI d3d9_vertexbuffer_FreePrivateData(IDirect3DVertexBuffer9 *iface, REFGUID guid)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
     TRACE("iface %p, guid %s.\n", iface, debugstr_guid(guid));
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DVertexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_free_private_data(resource, guid);
     wined3d_mutex_unlock();
 
@@ -157,13 +155,13 @@ static HRESULT WINAPI d3d9_vertexbuffer_FreePrivateData(IDirect3DVertexBuffer9 *
 
 static DWORD WINAPI d3d9_vertexbuffer_SetPriority(IDirect3DVertexBuffer9 *iface, DWORD priority)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
     DWORD previous;
 
     TRACE("iface %p, priority %u.\n", iface, priority);
 
     wined3d_mutex_lock();
-    previous = wined3d_buffer_set_priority(buffer->wineD3DVertexBuffer, priority);
+    previous = wined3d_buffer_set_priority(buffer->wined3d_buffer, priority);
     wined3d_mutex_unlock();
 
     return previous;
@@ -171,13 +169,13 @@ static DWORD WINAPI d3d9_vertexbuffer_SetPriority(IDirect3DVertexBuffer9 *iface,
 
 static DWORD WINAPI d3d9_vertexbuffer_GetPriority(IDirect3DVertexBuffer9 *iface)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
     DWORD priority;
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    priority = wined3d_buffer_get_priority(buffer->wineD3DVertexBuffer);
+    priority = wined3d_buffer_get_priority(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 
     return priority;
@@ -185,12 +183,12 @@ static DWORD WINAPI d3d9_vertexbuffer_GetPriority(IDirect3DVertexBuffer9 *iface)
 
 static void WINAPI d3d9_vertexbuffer_PreLoad(IDirect3DVertexBuffer9 *iface)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    wined3d_buffer_preload(buffer->wineD3DVertexBuffer);
+    wined3d_buffer_preload(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 }
 
@@ -204,14 +202,14 @@ static D3DRESOURCETYPE WINAPI d3d9_vertexbuffer_GetType(IDirect3DVertexBuffer9 *
 static HRESULT WINAPI d3d9_vertexbuffer_Lock(IDirect3DVertexBuffer9 *iface, UINT offset, UINT size,
         void **data, DWORD flags)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
     HRESULT hr;
 
     TRACE("iface %p, offset %u, size %u, data %p, flags %#x.\n",
             iface, offset, size, data, flags);
 
     wined3d_mutex_lock();
-    hr = wined3d_buffer_map(buffer->wineD3DVertexBuffer, offset, size, (BYTE **)data, flags);
+    hr = wined3d_buffer_map(buffer->wined3d_buffer, offset, size, (BYTE **)data, flags);
     wined3d_mutex_unlock();
 
     return hr;
@@ -219,12 +217,12 @@ static HRESULT WINAPI d3d9_vertexbuffer_Lock(IDirect3DVertexBuffer9 *iface, UINT
 
 static HRESULT WINAPI d3d9_vertexbuffer_Unlock(IDirect3DVertexBuffer9 *iface)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    wined3d_buffer_unmap(buffer->wineD3DVertexBuffer);
+    wined3d_buffer_unmap(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 
     return D3D_OK;
@@ -233,14 +231,14 @@ static HRESULT WINAPI d3d9_vertexbuffer_Unlock(IDirect3DVertexBuffer9 *iface)
 static HRESULT WINAPI d3d9_vertexbuffer_GetDesc(IDirect3DVertexBuffer9 *iface,
         D3DVERTEXBUFFER_DESC *desc)
 {
-    IDirect3DVertexBuffer9Impl *buffer = impl_from_IDirect3DVertexBuffer9(iface);
+    struct d3d9_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer9(iface);
     struct wined3d_resource_desc wined3d_desc;
     struct wined3d_resource *wined3d_resource;
 
     TRACE("iface %p, desc %p.\n", iface, desc);
 
     wined3d_mutex_lock();
-    wined3d_resource = wined3d_buffer_get_resource(buffer->wineD3DVertexBuffer);
+    wined3d_resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     wined3d_resource_get_desc(wined3d_resource, &wined3d_desc);
     wined3d_mutex_unlock();
 
@@ -285,18 +283,18 @@ static const struct wined3d_parent_ops d3d9_vertexbuffer_wined3d_parent_ops =
     d3d9_vertexbuffer_wined3d_object_destroyed,
 };
 
-HRESULT vertexbuffer_init(IDirect3DVertexBuffer9Impl *buffer, IDirect3DDevice9Impl *device,
+HRESULT vertexbuffer_init(struct d3d9_vertexbuffer *buffer, struct d3d9_device *device,
         UINT size, UINT usage, DWORD fvf, D3DPOOL pool)
 {
     HRESULT hr;
 
     buffer->IDirect3DVertexBuffer9_iface.lpVtbl = &d3d9_vertexbuffer_vtbl;
-    buffer->ref = 1;
+    buffer->refcount = 1;
     buffer->fvf = fvf;
 
     wined3d_mutex_lock();
     hr = wined3d_buffer_create_vb(device->wined3d_device, size, usage & WINED3DUSAGE_MASK,
-            (enum wined3d_pool)pool, buffer, &d3d9_vertexbuffer_wined3d_parent_ops, &buffer->wineD3DVertexBuffer);
+            (enum wined3d_pool)pool, buffer, &d3d9_vertexbuffer_wined3d_parent_ops, &buffer->wined3d_buffer);
     wined3d_mutex_unlock();
     if (FAILED(hr))
     {
@@ -304,13 +302,13 @@ HRESULT vertexbuffer_init(IDirect3DVertexBuffer9Impl *buffer, IDirect3DDevice9Im
         return hr;
     }
 
-    buffer->parentDevice = &device->IDirect3DDevice9Ex_iface;
-    IDirect3DDevice9Ex_AddRef(buffer->parentDevice);
+    buffer->parent_device = &device->IDirect3DDevice9Ex_iface;
+    IDirect3DDevice9Ex_AddRef(buffer->parent_device);
 
     return D3D_OK;
 }
 
-IDirect3DVertexBuffer9Impl *unsafe_impl_from_IDirect3DVertexBuffer9(IDirect3DVertexBuffer9 *iface)
+struct d3d9_vertexbuffer *unsafe_impl_from_IDirect3DVertexBuffer9(IDirect3DVertexBuffer9 *iface)
 {
     if (!iface)
         return NULL;
@@ -319,42 +317,42 @@ IDirect3DVertexBuffer9Impl *unsafe_impl_from_IDirect3DVertexBuffer9(IDirect3DVer
     return impl_from_IDirect3DVertexBuffer9(iface);
 }
 
-static inline IDirect3DIndexBuffer9Impl *impl_from_IDirect3DIndexBuffer9(IDirect3DIndexBuffer9 *iface)
+static inline struct d3d9_indexbuffer *impl_from_IDirect3DIndexBuffer9(IDirect3DIndexBuffer9 *iface)
 {
-    return CONTAINING_RECORD(iface, IDirect3DIndexBuffer9Impl, IDirect3DIndexBuffer9_iface);
+    return CONTAINING_RECORD(iface, struct d3d9_indexbuffer, IDirect3DIndexBuffer9_iface);
 }
 
-static HRESULT WINAPI d3d9_indexbuffer_QueryInterface(IDirect3DIndexBuffer9 *iface, REFIID riid, void **object)
+static HRESULT WINAPI d3d9_indexbuffer_QueryInterface(IDirect3DIndexBuffer9 *iface, REFIID riid, void **out)
 {
-    TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), object);
+    TRACE("iface %p, riid %s, out %p.\n", iface, debugstr_guid(riid), out);
 
     if (IsEqualGUID(riid, &IID_IDirect3DIndexBuffer9)
             || IsEqualGUID(riid, &IID_IDirect3DResource9)
             || IsEqualGUID(riid, &IID_IUnknown))
     {
         IDirect3DIndexBuffer9_AddRef(iface);
-        *object = iface;
+        *out = iface;
         return S_OK;
     }
 
     WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(riid));
 
-    *object = NULL;
+    *out = NULL;
     return E_NOINTERFACE;
 }
 
 static ULONG WINAPI d3d9_indexbuffer_AddRef(IDirect3DIndexBuffer9 *iface)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
-    ULONG refcount = InterlockedIncrement(&buffer->ref);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    ULONG refcount = InterlockedIncrement(&buffer->refcount);
 
     TRACE("%p increasing refcount to %u.\n", iface, refcount);
 
     if (refcount == 1)
     {
-        IDirect3DDevice9Ex_AddRef(buffer->parentDevice);
+        IDirect3DDevice9Ex_AddRef(buffer->parent_device);
         wined3d_mutex_lock();
-        wined3d_buffer_incref(buffer->wineD3DIndexBuffer);
+        wined3d_buffer_incref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
     }
 
@@ -363,17 +361,17 @@ static ULONG WINAPI d3d9_indexbuffer_AddRef(IDirect3DIndexBuffer9 *iface)
 
 static ULONG WINAPI d3d9_indexbuffer_Release(IDirect3DIndexBuffer9 *iface)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
-    ULONG refcount = InterlockedDecrement(&buffer->ref);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    ULONG refcount = InterlockedDecrement(&buffer->refcount);
 
     TRACE("%p decreasing refcount to %u.\n", iface, refcount);
 
     if (!refcount)
     {
-        IDirect3DDevice9Ex *device = buffer->parentDevice;
+        IDirect3DDevice9Ex *device = buffer->parent_device;
 
         wined3d_mutex_lock();
-        wined3d_buffer_decref(buffer->wineD3DIndexBuffer);
+        wined3d_buffer_decref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
 
         /* Release the device last, as it may cause the device to be destroyed. */
@@ -383,14 +381,13 @@ static ULONG WINAPI d3d9_indexbuffer_Release(IDirect3DIndexBuffer9 *iface)
     return refcount;
 }
 
-static HRESULT WINAPI d3d9_indexbuffer_GetDevice(IDirect3DIndexBuffer9 *iface,
-        IDirect3DDevice9 **device)
+static HRESULT WINAPI d3d9_indexbuffer_GetDevice(IDirect3DIndexBuffer9 *iface, IDirect3DDevice9 **device)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = (IDirect3DDevice9 *)buffer->parentDevice;
+    *device = (IDirect3DDevice9 *)buffer->parent_device;
     IDirect3DDevice9_AddRef(*device);
 
     TRACE("Returning device %p.\n", *device);
@@ -401,7 +398,7 @@ static HRESULT WINAPI d3d9_indexbuffer_GetDevice(IDirect3DIndexBuffer9 *iface,
 static HRESULT WINAPI d3d9_indexbuffer_SetPrivateData(IDirect3DIndexBuffer9 *iface,
         REFGUID guid, const void *data, DWORD data_size, DWORD flags)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
@@ -409,7 +406,7 @@ static HRESULT WINAPI d3d9_indexbuffer_SetPrivateData(IDirect3DIndexBuffer9 *ifa
             iface, debugstr_guid(guid), data, data_size, flags);
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DIndexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_set_private_data(resource, guid, data, data_size, flags);
     wined3d_mutex_unlock();
 
@@ -419,7 +416,7 @@ static HRESULT WINAPI d3d9_indexbuffer_SetPrivateData(IDirect3DIndexBuffer9 *ifa
 static HRESULT WINAPI d3d9_indexbuffer_GetPrivateData(IDirect3DIndexBuffer9 *iface,
         REFGUID guid, void *data, DWORD *data_size)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
@@ -427,7 +424,7 @@ static HRESULT WINAPI d3d9_indexbuffer_GetPrivateData(IDirect3DIndexBuffer9 *ifa
             iface, debugstr_guid(guid), data, data_size);
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DIndexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_get_private_data(resource, guid, data, data_size);
     wined3d_mutex_unlock();
 
@@ -436,14 +433,14 @@ static HRESULT WINAPI d3d9_indexbuffer_GetPrivateData(IDirect3DIndexBuffer9 *ifa
 
 static HRESULT WINAPI d3d9_indexbuffer_FreePrivateData(IDirect3DIndexBuffer9 *iface, REFGUID guid)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
     TRACE("iface %p, guid %s.\n", iface, debugstr_guid(guid));
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DIndexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_free_private_data(resource, guid);
     wined3d_mutex_unlock();
 
@@ -452,13 +449,13 @@ static HRESULT WINAPI d3d9_indexbuffer_FreePrivateData(IDirect3DIndexBuffer9 *if
 
 static DWORD WINAPI d3d9_indexbuffer_SetPriority(IDirect3DIndexBuffer9 *iface, DWORD priority)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
     DWORD previous;
 
     TRACE("iface %p, priority %u.\n", iface, priority);
 
     wined3d_mutex_lock();
-    previous = wined3d_buffer_set_priority(buffer->wineD3DIndexBuffer, priority);
+    previous = wined3d_buffer_set_priority(buffer->wined3d_buffer, priority);
     wined3d_mutex_unlock();
 
     return previous;
@@ -466,13 +463,13 @@ static DWORD WINAPI d3d9_indexbuffer_SetPriority(IDirect3DIndexBuffer9 *iface, D
 
 static DWORD WINAPI d3d9_indexbuffer_GetPriority(IDirect3DIndexBuffer9 *iface)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
     DWORD priority;
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    priority = wined3d_buffer_get_priority(buffer->wineD3DIndexBuffer);
+    priority = wined3d_buffer_get_priority(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 
     return priority;
@@ -480,12 +477,12 @@ static DWORD WINAPI d3d9_indexbuffer_GetPriority(IDirect3DIndexBuffer9 *iface)
 
 static void WINAPI d3d9_indexbuffer_PreLoad(IDirect3DIndexBuffer9 *iface)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    wined3d_buffer_preload(buffer->wineD3DIndexBuffer);
+    wined3d_buffer_preload(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 }
 
@@ -499,14 +496,14 @@ static D3DRESOURCETYPE WINAPI d3d9_indexbuffer_GetType(IDirect3DIndexBuffer9 *if
 static HRESULT WINAPI d3d9_indexbuffer_Lock(IDirect3DIndexBuffer9 *iface,
         UINT offset, UINT size, void **data, DWORD flags)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
     HRESULT hr;
 
     TRACE("iface %p, offset %u, size %u, data %p, flags %#x.\n",
             iface, offset, size, data, flags);
 
     wined3d_mutex_lock();
-    hr = wined3d_buffer_map(buffer->wineD3DIndexBuffer, offset, size, (BYTE **)data, flags);
+    hr = wined3d_buffer_map(buffer->wined3d_buffer, offset, size, (BYTE **)data, flags);
     wined3d_mutex_unlock();
 
     return hr;
@@ -514,28 +511,27 @@ static HRESULT WINAPI d3d9_indexbuffer_Lock(IDirect3DIndexBuffer9 *iface,
 
 static HRESULT WINAPI d3d9_indexbuffer_Unlock(IDirect3DIndexBuffer9 *iface)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    wined3d_buffer_unmap(buffer->wineD3DIndexBuffer);
+    wined3d_buffer_unmap(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 
     return D3D_OK;
 }
 
-static HRESULT WINAPI d3d9_indexbuffer_GetDesc(IDirect3DIndexBuffer9 *iface,
-        D3DINDEXBUFFER_DESC *desc)
+static HRESULT WINAPI d3d9_indexbuffer_GetDesc(IDirect3DIndexBuffer9 *iface, D3DINDEXBUFFER_DESC *desc)
 {
-    IDirect3DIndexBuffer9Impl *buffer = impl_from_IDirect3DIndexBuffer9(iface);
+    struct d3d9_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer9(iface);
     struct wined3d_resource_desc wined3d_desc;
     struct wined3d_resource *wined3d_resource;
 
     TRACE("iface %p, desc %p.\n", iface, desc);
 
     wined3d_mutex_lock();
-    wined3d_resource = wined3d_buffer_get_resource(buffer->wineD3DIndexBuffer);
+    wined3d_resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     wined3d_resource_get_desc(wined3d_resource, &wined3d_desc);
     wined3d_mutex_unlock();
 
@@ -579,18 +575,18 @@ static const struct wined3d_parent_ops d3d9_indexbuffer_wined3d_parent_ops =
     d3d9_indexbuffer_wined3d_object_destroyed,
 };
 
-HRESULT indexbuffer_init(IDirect3DIndexBuffer9Impl *buffer, IDirect3DDevice9Impl *device,
+HRESULT indexbuffer_init(struct d3d9_indexbuffer *buffer, struct d3d9_device *device,
         UINT size, DWORD usage, D3DFORMAT format, D3DPOOL pool)
 {
     HRESULT hr;
 
     buffer->IDirect3DIndexBuffer9_iface.lpVtbl = &d3d9_indexbuffer_vtbl;
-    buffer->ref = 1;
+    buffer->refcount = 1;
     buffer->format = wined3dformat_from_d3dformat(format);
 
     wined3d_mutex_lock();
     hr = wined3d_buffer_create_ib(device->wined3d_device, size, usage & WINED3DUSAGE_MASK,
-            (enum wined3d_pool)pool, buffer, &d3d9_indexbuffer_wined3d_parent_ops, &buffer->wineD3DIndexBuffer);
+            (enum wined3d_pool)pool, buffer, &d3d9_indexbuffer_wined3d_parent_ops, &buffer->wined3d_buffer);
     wined3d_mutex_unlock();
     if (FAILED(hr))
     {
@@ -598,13 +594,13 @@ HRESULT indexbuffer_init(IDirect3DIndexBuffer9Impl *buffer, IDirect3DDevice9Impl
         return hr;
     }
 
-    buffer->parentDevice = &device->IDirect3DDevice9Ex_iface;
-    IDirect3DDevice9Ex_AddRef(buffer->parentDevice);
+    buffer->parent_device = &device->IDirect3DDevice9Ex_iface;
+    IDirect3DDevice9Ex_AddRef(buffer->parent_device);
 
     return D3D_OK;
 }
 
-IDirect3DIndexBuffer9Impl *unsafe_impl_from_IDirect3DIndexBuffer9(IDirect3DIndexBuffer9 *iface)
+struct d3d9_indexbuffer *unsafe_impl_from_IDirect3DIndexBuffer9(IDirect3DIndexBuffer9 *iface)
 {
     if (!iface)
         return NULL;
