@@ -59,24 +59,24 @@ static HRESULT DEVENUM_CreateSpecialCategories(void);
 /**********************************************************************
  * DEVENUM_ICreateDevEnum_QueryInterface (also IUnknown)
  */
-static HRESULT WINAPI DEVENUM_ICreateDevEnum_QueryInterface(
-    ICreateDevEnum * iface,
-    REFIID riid,
-    LPVOID *ppvObj)
+static HRESULT WINAPI DEVENUM_ICreateDevEnum_QueryInterface(ICreateDevEnum *iface, REFIID riid,
+        void **ppv)
 {
-    TRACE("\n\tIID:\t%s\n",debugstr_guid(riid));
+    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(riid), ppv);
 
-    if (ppvObj == NULL) return E_POINTER;
+    if (!ppv)
+        return E_POINTER;
 
     if (IsEqualGUID(riid, &IID_IUnknown) ||
 	IsEqualGUID(riid, &IID_ICreateDevEnum))
     {
-        *ppvObj = iface;
+        *ppv = iface;
 	DEVENUM_ICreateDevEnum_AddRef(iface);
 	return S_OK;
     }
 
     FIXME("- no interface IID: %s\n", debugstr_guid(riid));
+    *ppv = NULL;
     return E_NOINTERFACE;
 }
 
@@ -171,10 +171,10 @@ static void DEVENUM_ReadPinTypes(HKEY hkeyPinKey, REGFILTERPINS *rgPin)
         for (i1 = 0; i1 < dwMinorTypes; i1++)
         {
             WCHAR wszMinorTypeName[64];
-            DWORD cName = sizeof(wszMinorTypeName) / sizeof(WCHAR);
             CLSID *clsMajorType = NULL, *clsMinorType = NULL;
             HRESULT hr;
 
+            cName = sizeof(wszMinorTypeName) / sizeof(WCHAR);
             if (RegEnumKeyExW(hkeyMajorType, i1, wszMinorTypeName, &cName, NULL, NULL, NULL, NULL) != ERROR_SUCCESS) continue;
 
             clsMinorType = CoTaskMemAlloc(sizeof(CLSID));
@@ -258,7 +258,7 @@ static void DEVENUM_ReadPins(HKEY hkeyFilterClass, REGFILTER2 *rgf2)
         WCHAR wszPinName[MAX_PATH];
         DWORD cName = sizeof(wszPinName) / sizeof(WCHAR);
         DWORD Type, cbData;
-        REGFILTERPINS *rgPin = &rgPins[rgf2->u.s.cPins];
+        REGFILTERPINS *rgPin = &rgPins[rgf2->u.s1.cPins];
         LONG lRet;
 
         rgPin->strName = NULL;
@@ -298,7 +298,7 @@ static void DEVENUM_ReadPins(HKEY hkeyFilterClass, REGFILTER2 *rgf2)
 
         DEVENUM_ReadPinTypes(hkeyPinKey, rgPin);
 
-        ++rgf2->u.s.cPins;
+        ++rgf2->u.s1.cPins;
         continue;
 
         error_cleanup:
@@ -309,13 +309,13 @@ static void DEVENUM_ReadPins(HKEY hkeyFilterClass, REGFILTER2 *rgf2)
 
     RegCloseKey(hkeyPins);
 
-    if (rgPins && !rgf2->u.s.cPins)
+    if (rgPins && !rgf2->u.s1.cPins)
     {
         CoTaskMemFree(rgPins);
         rgPins = NULL;
     }
 
-    rgf2->u.s.rgPins = rgPins;
+    rgf2->u.s1.rgPins = rgPins;
 }
 
 static HRESULT DEVENUM_RegisterLegacyAmFilters(void)
@@ -349,7 +349,6 @@ static HRESULT DEVENUM_RegisterLegacyAmFilters(void)
             HKEY hkeyCategoryBaseKey;
             WCHAR wszRegKey[MAX_PATH];
             HKEY hkeyInstance = NULL;
-            HRESULT hr;
 
             if (RegEnumKeyExW(hkeyFilter, i, wszFilterSubkeyName, &cName, NULL, NULL, NULL, NULL) != ERROR_SUCCESS) continue;
 
@@ -387,8 +386,8 @@ static HRESULT DEVENUM_RegisterLegacyAmFilters(void)
 
                 rgf2.dwVersion = 1;
                 rgf2.dwMerit = 0;
-                rgf2.u.s.cPins = 0;
-                rgf2.u.s.rgPins = NULL;
+                rgf2.u.s1.cPins = 0;
+                rgf2.u.s1.rgPins = NULL;
 
                 cbData = sizeof(wszFilterName);
                 if (RegQueryValueExW(hkeyFilterClass, NULL, NULL, &Type, (LPBYTE)wszFilterName, &cbData) != ERROR_SUCCESS ||
@@ -414,29 +413,29 @@ static HRESULT DEVENUM_RegisterLegacyAmFilters(void)
 
                 if (hkeyFilterClass) RegCloseKey(hkeyFilterClass);
 
-                if (rgf2.u.s.rgPins)
+                if (rgf2.u.s1.rgPins)
                 {
                     UINT iPin;
 
-                    for (iPin = 0; iPin < rgf2.u.s.cPins; iPin++)
+                    for (iPin = 0; iPin < rgf2.u.s1.cPins; iPin++)
                     {
-                        CoTaskMemFree(rgf2.u.s.rgPins[iPin].strName);
+                        CoTaskMemFree(rgf2.u.s1.rgPins[iPin].strName);
 
-                        if (rgf2.u.s.rgPins[iPin].lpMediaType)
+                        if (rgf2.u.s1.rgPins[iPin].lpMediaType)
                         {
                             UINT iType;
 
-                            for (iType = 0; iType < rgf2.u.s.rgPins[iPin].nMediaTypes; iType++)
+                            for (iType = 0; iType < rgf2.u.s1.rgPins[iPin].nMediaTypes; iType++)
                             {
-                                CoTaskMemFree((void*)rgf2.u.s.rgPins[iPin].lpMediaType[iType].clsMajorType);
-                                CoTaskMemFree((void*)rgf2.u.s.rgPins[iPin].lpMediaType[iType].clsMinorType);
+                                CoTaskMemFree((void*)rgf2.u.s1.rgPins[iPin].lpMediaType[iType].clsMajorType);
+                                CoTaskMemFree((void*)rgf2.u.s1.rgPins[iPin].lpMediaType[iType].clsMinorType);
                             }
 
-                            CoTaskMemFree((void*)rgf2.u.s.rgPins[iPin].lpMediaType);
+                            CoTaskMemFree((void*)rgf2.u.s1.rgPins[iPin].lpMediaType);
                         }
                     }
 
-                    CoTaskMemFree((void*)rgf2.u.s.rgPins);
+                    CoTaskMemFree((void*)rgf2.u.s1.rgPins);
                 }
             }
         }
@@ -463,9 +462,8 @@ static HRESULT WINAPI DEVENUM_ICreateDevEnum_CreateClassEnumerator(
     HKEY hkey;
     HKEY hbasekey;
     HRESULT hr;
-    CreateDevEnumImpl *This = (CreateDevEnumImpl *)iface;
 
-    TRACE("(%p)->(%s, %p, %x)\n\tDeviceClass:\t%s\n", This, debugstr_guid(clsidDeviceClass), ppEnumMoniker, dwFlags, debugstr_guid(clsidDeviceClass));
+    TRACE("(%p)->(%s, %p, %x)\n", iface, debugstr_guid(clsidDeviceClass), ppEnumMoniker, dwFlags);
 
     if (!ppEnumMoniker)
         return E_POINTER;
@@ -519,7 +517,7 @@ static const ICreateDevEnumVtbl ICreateDevEnum_Vtbl =
 /**********************************************************************
  * static CreateDevEnum instance
  */
-CreateDevEnumImpl DEVENUM_CreateDevEnum = { &ICreateDevEnum_Vtbl };
+ICreateDevEnum DEVENUM_CreateDevEnum = { &ICreateDevEnum_Vtbl };
 
 /**********************************************************************
  * DEVENUM_CreateAMCategoryKey (INTERNAL)
@@ -606,8 +604,8 @@ static HRESULT DEVENUM_CreateSpecialCategories(void)
 
     rf2.dwVersion = 2;
     rf2.dwMerit = MERIT_PREFERRED;
-    rf2.u.s1.cPins2 = 1;
-    rf2.u.s1.rgPins2 = &rfp2;
+    rf2.u.s2.cPins2 = 1;
+    rf2.u.s2.rgPins2 = &rfp2;
     rfp2.cInstances = 1;
     rfp2.nMediums = 0;
     rfp2.lpMedium = NULL;
