@@ -22,7 +22,6 @@
 #include "winerror.h"
 #include "wine/debug.h"
 
-#include "dpinit.h"
 #include "wine/dplaysp.h"
 #include "dplay_global.h"
 #include "name_server.h"
@@ -37,7 +36,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(dplay);
 /* Prototypes */
 static BOOL DPSP_CreateIUnknown( LPVOID lpSP );
 static BOOL DPSP_DestroyIUnknown( LPVOID lpSP );
-static BOOL DPSP_CreateDirectPlaySP( LPVOID lpSP, IDirectPlay2Impl* dp );
+static BOOL DPSP_CreateDirectPlaySP( void *lpSP, IDirectPlayImpl *dp );
 static BOOL DPSP_DestroyDirectPlaySP( LPVOID lpSP );
 
 /* Predefine the interface */
@@ -57,7 +56,7 @@ typedef struct tagDirectPlaySPData
   LPVOID lpSpLocalData;
   DWORD  dwSpLocalDataSize; /* Size of data pointed to by lpSpLocalData */
 
-  IDirectPlay2Impl* dplay; /* FIXME: This should perhaps be iface not impl */
+  IDirectPlayImpl *dplay; /* FIXME: This should perhaps be iface not impl */
 
 } DirectPlaySPData;
 
@@ -86,7 +85,7 @@ typedef struct tagDP_SPPLAYERDATA
 } DP_SPPLAYERDATA, *LPDP_SPPLAYERDATA;
 
 /* Create the SP interface */
-HRESULT DPSP_CreateInterface( REFIID riid, LPVOID* ppvObj, IDirectPlay2Impl* dp )
+HRESULT DPSP_CreateInterface( REFIID riid, void **ppvObj, IDirectPlayImpl *dp )
 {
   TRACE( " for %s\n", debugstr_guid( riid ) );
 
@@ -100,7 +99,7 @@ HRESULT DPSP_CreateInterface( REFIID riid, LPVOID* ppvObj, IDirectPlay2Impl* dp
 
   if( IsEqualGUID( &IID_IDirectPlaySP, riid ) )
   {
-    IDirectPlaySPImpl *This = (IDirectPlaySPImpl *)*ppvObj;
+    IDirectPlaySPImpl *This = *ppvObj;
     This->lpVtbl = &directPlaySPVT;
   }
   else
@@ -133,7 +132,7 @@ HRESULT DPSP_CreateInterface( REFIID riid, LPVOID* ppvObj, IDirectPlay2Impl* dp
 
 static BOOL DPSP_CreateIUnknown( LPVOID lpSP )
 {
-  IDirectPlaySPImpl *This = (IDirectPlaySPImpl *)lpSP;
+  IDirectPlaySPImpl *This = lpSP;
 
   This->unk = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof( *(This->unk) ) );
 
@@ -150,7 +149,7 @@ static BOOL DPSP_CreateIUnknown( LPVOID lpSP )
 
 static BOOL DPSP_DestroyIUnknown( LPVOID lpSP )
 {
-  IDirectPlaySPImpl *This = (IDirectPlaySPImpl *)lpSP;
+  IDirectPlaySPImpl *This = lpSP;
 
   This->unk->DPSP_lock.DebugInfo->Spare[0] = 0;
   DeleteCriticalSection( &This->unk->DPSP_lock );
@@ -160,9 +159,9 @@ static BOOL DPSP_DestroyIUnknown( LPVOID lpSP )
 }
 
 
-static BOOL DPSP_CreateDirectPlaySP( LPVOID lpSP, IDirectPlay2Impl* dp )
+static BOOL DPSP_CreateDirectPlaySP( void *lpSP, IDirectPlayImpl *dp )
 {
-  IDirectPlaySPImpl *This = (IDirectPlaySPImpl *)lpSP;
+  IDirectPlaySPImpl *This = lpSP;
 
   This->sp = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof( *(This->sp) ) );
 
@@ -173,35 +172,12 @@ static BOOL DPSP_CreateDirectPlaySP( LPVOID lpSP, IDirectPlay2Impl* dp )
 
   This->sp->dplay = dp;
 
-  /* Normally we should be keeping a reference, but since only the dplay
-   * interface that created us can destroy us, we do not keep a reference
-   * to it (ie we'd be stuck with always having one reference to the dplay
-   * object, and hence us, around).
-   * NOTE: The dp object does reference count us.
-   *
-   * FIXME: This is a kludge to get around a problem where a queryinterface
-   *        is used to get a new interface and then is closed. We will then
-   *        reference garbage. However, with this we will never deallocate
-   *        the interface we store. The correct fix is to require all
-   *        DP internal interfaces to use the This->dp2 interface which
-   *        should be changed to This->dp
-   */
-  IDirectPlayX_AddRef( (LPDIRECTPLAY2)dp );
-
   return TRUE;
 }
 
 static BOOL DPSP_DestroyDirectPlaySP( LPVOID lpSP )
 {
-  IDirectPlaySPImpl *This = (IDirectPlaySPImpl *)lpSP;
-
-  /* Normally we should be keeping a reference, but since only the dplay
-   * interface that created us can destroy us, we do not keep a reference
-   * to it (ie we'd be stuck with always having one reference to the dplay
-   * object, and hence us, around).
-   * NOTE: The dp object does reference count us.
-   */
-  /*IDirectPlayX_Release( (LPDIRECTPLAY2)This->sp->dplay ); */
+  IDirectPlaySPImpl *This = lpSP;
 
   HeapFree( GetProcessHeap(), 0, This->sp->lpSpRemoteData );
   HeapFree( GetProcessHeap(), 0, This->sp->lpSpLocalData );
@@ -235,7 +211,7 @@ static HRESULT WINAPI DPSP_QueryInterface
 
   if( IsEqualGUID( &IID_IDirectPlaySP, riid ) )
   {
-    IDirectPlaySPImpl *This = (IDirectPlaySPImpl *)*ppvObj;
+    IDirectPlaySPImpl *This = *ppvObj;
     This->lpVtbl = &directPlaySPVT;
   }
   else
@@ -434,7 +410,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
   LPVOID lpMessageHeader
 )
 {
-  LPDPMSG_SENDENVELOPE lpMsg = (LPDPMSG_SENDENVELOPE)lpMessageBody;
+  LPDPMSG_SENDENVELOPE lpMsg = lpMessageBody;
   HRESULT hr = DPERR_GENERIC;
   WORD wCommandId;
   WORD wVersion;
@@ -459,7 +435,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
 #if 0
   {
-    const LPDWORD lpcHeader = (LPDWORD)lpMessageHeader;
+    const LPDWORD lpcHeader = lpMessageHeader;
 
     TRACE( "lpMessageHeader = [0x%08lx] [0x%08lx] [0x%08lx] [0x%08lx] [0x%08lx]\n",
            lpcHeader[0], lpcHeader[1], lpcHeader[2], lpcHeader[3], lpcHeader[4] );
@@ -509,7 +485,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
   {
     case DPSYS_CREATEPLAYERORGROUP:
     {
-      LPDPMSG_CREATEPLAYERORGROUP msg = (LPDPMSG_CREATEPLAYERORGROUP)lpMsg;
+      LPDPMSG_CREATEPLAYERORGROUP msg = lpMsg;
 
       if( msg->dwPlayerType == DPPLAYERTYPE_PLAYER )
       {
@@ -544,7 +520,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_DESTROYPLAYERORGROUP:
     {
-      LPDPMSG_DESTROYPLAYERORGROUP msg = (LPDPMSG_DESTROYPLAYERORGROUP)lpMsg;
+      LPDPMSG_DESTROYPLAYERORGROUP msg = lpMsg;
 
       if( msg->dwPlayerType == DPPLAYERTYPE_PLAYER )
       {
@@ -565,7 +541,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_ADDPLAYERTOGROUP:
     {
-      LPDPMSG_ADDPLAYERTOGROUP msg = (LPDPMSG_ADDPLAYERTOGROUP)lpMsg;
+      LPDPMSG_ADDPLAYERTOGROUP msg = lpMsg;
 
       hr = DP_IF_AddPlayerToGroup( This, msg->dpIdGroup, msg->dpIdPlayer, ... );
       break;
@@ -573,7 +549,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_DELETEPLAYERFROMGROUP:
     {
-      LPDPMSG_DELETEPLAYERFROMGROUP msg = (LPDPMSG_DELETEPLAYERFROMGROUP)lpMsg;
+      LPDPMSG_DELETEPLAYERFROMGROUP msg = lpMsg;
 
       hr = DP_IF_DeletePlayerFromGroup( This, msg->dpIdGroup, msg->dpIdPlayer,
                                         ... );
@@ -583,7 +559,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_SESSIONLOST:
     {
-      LPDPMSG_SESSIONLOST msg = (LPDPMSG_SESSIONLOST)lpMsg;
+      LPDPMSG_SESSIONLOST msg = lpMsg;
 
       FIXME( "DPSYS_SESSIONLOST not handled\n" );
 
@@ -592,7 +568,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_HOST:
     {
-      LPDPMSG_HOST msg = (LPDPMSG_HOST)lpMsg;
+      LPDPMSG_HOST msg = lpMsg;
 
       FIXME( "DPSYS_HOST not handled\n" );
 
@@ -601,7 +577,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_SETPLAYERORGROUPDATA:
     {
-      LPDPMSG_SETPLAYERORGROUPDATA msg = (LPDPMSG_SETPLAYERORGROUPDATA)lpMsg;
+      LPDPMSG_SETPLAYERORGROUPDATA msg = lpMsg;
 
       if( msg->dwPlayerType == DPPLAYERTYPE_PLAYER )
       {
@@ -623,7 +599,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_SETPLAYERORGROUPNAME:
     {
-      LPDPMSG_SETPLAYERORGROUPNAME msg = (LPDPMSG_SETPLAYERORGROUPNAME)lpMsg;
+      LPDPMSG_SETPLAYERORGROUPNAME msg = lpMsg;
 
       if( msg->dwPlayerType == DPPLAYERTYPE_PLAYER )
       {
@@ -644,7 +620,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_SETSESSIONDESC;
     {
-      LPDPMSG_SETSESSIONDESC msg = (LPDPMSG_SETSESSIONDESC)lpMsg;
+      LPDPMSG_SETSESSIONDESC msg = lpMsg;
 
       hr = DP_IF_SetSessionDesc( This, &msg->dpDesc );
 
@@ -653,7 +629,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_ADDGROUPTOGROUP:
     {
-      LPDPMSG_ADDGROUPTOGROUP msg = (LPDPMSG_ADDGROUPTOGROUP)lpMsg;
+      LPDPMSG_ADDGROUPTOGROUP msg = lpMsg;
 
       hr = DP_IF_AddGroupToGroup( This, msg->dpIdParentGroup, msg->dpIdGroup,
                                   ... );
@@ -663,7 +639,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_DELETEGROUPFROMGROUP:
     {
-      LPDPMSG_DELETEGROUPFROMGROUP msg = (LPDPMSG_DELETEGROUPFROMGROUP)lpMsg;
+      LPDPMSG_DELETEGROUPFROMGROUP msg = lpMsg;
 
       hr = DP_IF_DeleteGroupFromGroup( This, msg->dpIdParentGroup,
                                        msg->dpIdGroup, ... );
@@ -673,7 +649,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_SECUREMESSAGE:
     {
-      LPDPMSG_SECUREMESSAGE msg = (LPDPMSG_SECUREMESSAGE)lpMsg;
+      LPDPMSG_SECUREMESSAGE msg = lpMsg;
 
       FIXME( "DPSYS_SECUREMESSAGE not implemented\n" );
 
@@ -682,7 +658,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_STARTSESSION:
     {
-      LPDPMSG_STARTSESSION msg = (LPDPMSG_STARTSESSION)lpMsg;
+      LPDPMSG_STARTSESSION msg = lpMsg;
 
       FIXME( "DPSYS_STARTSESSION not implemented\n" );
 
@@ -691,7 +667,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_CHAT:
     {
-      LPDPMSG_CHAT msg = (LPDPMSG_CHAT)lpMsg;
+      LPDPMSG_CHAT msg = lpMsg;
 
       FIXME( "DPSYS_CHAT not implemeneted\n" );
 
@@ -700,7 +676,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_SETGROUPOWNER:
     {
-      LPDPMSG_SETGROUPOWNER msg = (LPDPMSG_SETGROUPOWNER)lpMsg;
+      LPDPMSG_SETGROUPOWNER msg = lpMsg;
 
       FIXME( "DPSYS_SETGROUPOWNER not implemented\n" );
 
@@ -709,7 +685,7 @@ static HRESULT WINAPI IDirectPlaySPImpl_HandleMessage
 
     case DPSYS_SENDCOMPLETE:
     {
-      LPDPMSG_SENDCOMPLETE msg = (LPDPMSG_SENDCOMPLETE)lpMsg;
+      LPDPMSG_SENDCOMPLETE msg = lpMsg;
 
       FIXME( "DPSYS_SENDCOMPLETE not implemented\n" );
 
