@@ -28,7 +28,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(jscript);
 typedef struct {
     jsdisp_t dispex;
 
-    JSRegExp *jsregexp;
+    regexp_t *jsregexp;
     jsstr_t *str;
     INT last_index;
     jsval_t last_index_val;
@@ -58,7 +58,6 @@ static const WCHAR idx7W[] = {'$','7',0};
 static const WCHAR idx8W[] = {'$','8',0};
 static const WCHAR idx9W[] = {'$','9',0};
 
-static const WCHAR undefinedW[] = {'u','n','d','e','f','i','n','e','d',0};
 static const WCHAR emptyW[] = {0};
 
 static inline RegExpInstance *regexp_from_vdisp(vdisp_t *vdisp)
@@ -73,16 +72,13 @@ static void set_last_index(RegExpInstance *This, DWORD last_index)
     This->last_index_val = jsval_number(last_index);
 }
 
-static HRESULT do_regexp_match_next(script_ctx_t *ctx, RegExpInstance *regexp, DWORD rem_flags,
-        jsstr_t *str, const WCHAR **cp, match_result_t **parens, DWORD *parens_size,
-        DWORD *parens_cnt, match_result_t *ret)
+static HRESULT do_regexp_match_next(script_ctx_t *ctx, RegExpInstance *regexp,
+        DWORD rem_flags, jsstr_t *jsstr, const WCHAR *str, match_state_t *ret)
 {
-    REMatchState *result;
-    DWORD matchlen;
     HRESULT hres;
 
-    hres = MatchRegExpNext(regexp->jsregexp, str->str, jsstr_length(str),
-            cp, &ctx->tmp_heap, &result, &matchlen);
+    hres = regexp_execute(regexp->jsregexp, ctx, &ctx->tmp_heap,
+            str, jsstr_length(jsstr), ret);
     if(FAILED(hres))
         return hres;
     if(hres == S_FALSE) {
@@ -91,53 +87,21 @@ static HRESULT do_regexp_match_next(script_ctx_t *ctx, RegExpInstance *regexp, D
         return S_FALSE;
     }
 
-    if(parens) {
-        if(regexp->jsregexp->parenCount > *parens_size) {
-            match_result_t *new_parens;
-
-            if(*parens)
-                new_parens = heap_realloc(*parens, sizeof(match_result_t)*regexp->jsregexp->parenCount);
-            else
-                new_parens = heap_alloc(sizeof(match_result_t)*regexp->jsregexp->parenCount);
-            if(!new_parens)
-                return E_OUTOFMEMORY;
-
-            *parens_size = regexp->jsregexp->parenCount;
-            *parens = new_parens;
-        }
-    }
-
-    if(!(rem_flags & REM_NO_CTX_UPDATE) && ctx->last_match != str) {
+    if(!(rem_flags & REM_NO_CTX_UPDATE) && ctx->last_match != jsstr) {
         jsstr_release(ctx->last_match);
-        ctx->last_match = jsstr_addref(str);
-    }
-
-    if(parens) {
-        DWORD i;
-
-        *parens_cnt = regexp->jsregexp->parenCount;
-
-        for(i=0; i < regexp->jsregexp->parenCount; i++) {
-            if(result->parens[i].index == -1) {
-                (*parens)[i].str = NULL;
-                (*parens)[i].len = 0;
-            }else {
-                (*parens)[i].str = str->str + result->parens[i].index;
-                (*parens)[i].len = result->parens[i].length;
-            }
-        }
+        ctx->last_match = jsstr_addref(jsstr);
     }
 
     if(!(rem_flags & REM_NO_CTX_UPDATE)) {
-        DWORD i, n = min(sizeof(ctx->match_parens)/sizeof(ctx->match_parens[0]), regexp->jsregexp->parenCount);
+        DWORD i, n = min(sizeof(ctx->match_parens)/sizeof(ctx->match_parens[0]), ret->paren_count);
 
         for(i=0; i < n; i++) {
-            if(result->parens[i].index == -1) {
-                ctx->match_parens[i].str = NULL;
-                ctx->match_parens[i].len = 0;
+            if(ret->parens[i].index == -1) {
+                ctx->match_parens[i].index = 0;
+                ctx->match_parens[i].length = 0;
             }else {
-                ctx->match_parens[i].str = ctx->last_match->str + result->parens[i].index;
-                ctx->match_parens[i].len = result->parens[i].length;
+                ctx->match_parens[i].index = ret->parens[i].index;
+                ctx->match_parens[i].length = ret->parens[i].length;
             }
         }
 
@@ -145,51 +109,98 @@ static HRESULT do_regexp_match_next(script_ctx_t *ctx, RegExpInstance *regexp, D
             memset(ctx->match_parens+n, 0, sizeof(ctx->match_parens) - n*sizeof(ctx->match_parens[0]));
     }
 
-    ret->str = result->cp-matchlen;
-    ret->len = matchlen;
-    set_last_index(regexp, result->cp-str->str);
+    set_last_index(regexp, ret->cp-str);
 
     if(!(rem_flags & REM_NO_CTX_UPDATE)) {
-        ctx->last_match_index = ret->str-str->str;
-        ctx->last_match_length = matchlen;
+        ctx->last_match_index = ret->cp-str-ret->match_len;
+        ctx->last_match_length = ret->match_len;
     }
 
     return S_OK;
 }
 
-HRESULT regexp_match_next(script_ctx_t *ctx, jsdisp_t *dispex, DWORD rem_flags, jsstr_t *str,
-        const WCHAR **cp, match_result_t **parens, DWORD *parens_size, DWORD *parens_cnt,
-        match_result_t *ret)
+HRESULT regexp_match_next(script_ctx_t *ctx, jsdisp_t *dispex,
+        DWORD rem_flags, jsstr_t *jsstr, match_state_t **ret)
 {
     RegExpInstance *regexp = (RegExpInstance*)dispex;
+    match_state_t *match;
     heap_pool_t *mark;
+    const WCHAR *str;
     HRESULT hres;
 
-    if((rem_flags & REM_CHECK_GLOBAL) && !(regexp->jsregexp->flags & JSREG_GLOB))
+    if((rem_flags & REM_CHECK_GLOBAL) && !(regexp->jsregexp->flags & REG_GLOB)) {
+        if(rem_flags & REM_ALLOC_RESULT)
+            *ret = NULL;
         return S_FALSE;
+    }
+
+    str = jsstr_flatten(jsstr);
+    if(!str)
+        return E_OUTOFMEMORY;
+
+    if(rem_flags & REM_ALLOC_RESULT) {
+        match = alloc_match_state(regexp->jsregexp, NULL, str);
+        if(!match)
+            return E_OUTOFMEMORY;
+        *ret = match;
+    }
 
     mark = heap_pool_mark(&ctx->tmp_heap);
 
-    hres = do_regexp_match_next(ctx, regexp, rem_flags, str, cp, parens, parens_size, parens_cnt, ret);
+    if(rem_flags & REM_NO_PARENS) {
+        match = alloc_match_state(regexp->jsregexp, &ctx->tmp_heap, NULL);
+        if(!match) {
+            heap_pool_clear(mark);
+            return E_OUTOFMEMORY;
+        }
+        match->cp = (*ret)->cp;
+        match->match_len = (*ret)->match_len;
+    }else {
+        match = *ret;
+    }
+
+    hres = do_regexp_match_next(ctx, regexp, rem_flags, jsstr, str, match);
+
+    if(rem_flags & REM_NO_PARENS) {
+        (*ret)->cp = match->cp;
+        (*ret)->match_len = match->match_len;
+    }
 
     heap_pool_clear(mark);
+
+    if(hres != S_OK && (rem_flags & REM_ALLOC_RESULT)) {
+        heap_free(match);
+        *ret = NULL;
+    }
+
     return hres;
 }
 
-static HRESULT regexp_match(script_ctx_t *ctx, jsdisp_t *dispex, jsstr_t *str, BOOL gflag,
+static HRESULT regexp_match(script_ctx_t *ctx, jsdisp_t *dispex, jsstr_t *jsstr, BOOL gflag,
         match_result_t **match_result, DWORD *result_cnt)
 {
     RegExpInstance *This = (RegExpInstance*)dispex;
-    match_result_t *ret = NULL, cres;
-    const WCHAR *cp = str->str;
+    match_result_t *ret = NULL;
+    match_state_t *result;
     DWORD i=0, ret_size = 0;
     heap_pool_t *mark;
+    const WCHAR *str;
     HRESULT hres;
 
     mark = heap_pool_mark(&ctx->tmp_heap);
 
+    str = jsstr_flatten(jsstr);
+    if(!str)
+        return E_OUTOFMEMORY;
+
+    result = alloc_match_state(This->jsregexp, &ctx->tmp_heap, str);
+    if(!result) {
+        heap_pool_clear(mark);
+        return E_OUTOFMEMORY;
+    }
+
     while(1) {
-        hres = do_regexp_match_next(ctx, This, 0, str, &cp, NULL, NULL, NULL, &cres);
+        hres = do_regexp_match_next(ctx, This, 0, jsstr, str, result);
         if(hres == S_FALSE) {
             hres = S_OK;
             break;
@@ -214,9 +225,10 @@ static HRESULT regexp_match(script_ctx_t *ctx, jsdisp_t *dispex, jsstr_t *str, B
             }
         }
 
-        ret[i++] = cres;
+        ret[i].index = result->cp - str - result->match_len;
+        ret[i++].length = result->match_len;
 
-        if(!gflag && !(This->jsregexp->flags & JSREG_GLOB)) {
+        if(!gflag && !(This->jsregexp->flags & REG_GLOB)) {
             hres = S_OK;
             break;
         }
@@ -325,9 +337,10 @@ static HRESULT RegExp_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     return E_NOTIMPL;
 }
 
-static HRESULT create_match_array(script_ctx_t *ctx, jsstr_t *input, const match_result_t *result,
-        const match_result_t *parens, DWORD parens_cnt, IDispatch **ret)
+static HRESULT create_match_array(script_ctx_t *ctx, jsstr_t *input_str,
+        const match_state_t *result, IDispatch **ret)
 {
+    const WCHAR *input;
     jsdisp_t *array;
     jsstr_t *str;
     DWORD i;
@@ -338,12 +351,19 @@ static HRESULT create_match_array(script_ctx_t *ctx, jsstr_t *input, const match
     static const WCHAR lastIndexW[] = {'l','a','s','t','I','n','d','e','x',0};
     static const WCHAR zeroW[] = {'0',0};
 
-    hres = create_array(ctx, parens_cnt+1, &array);
+    input = jsstr_flatten(input_str);
+    if(!input)
+        return E_OUTOFMEMORY;
+
+    hres = create_array(ctx, result->paren_count+1, &array);
     if(FAILED(hres))
         return hres;
 
-    for(i=0; i < parens_cnt; i++) {
-        str = jsstr_alloc_len(parens[i].str, parens[i].len);
+    for(i=0; i < result->paren_count; i++) {
+        if(result->parens[i].index != -1)
+            str = jsstr_substr(input_str, result->parens[i].index, result->parens[i].length);
+        else
+            str = jsstr_empty();
         if(!str) {
             hres = E_OUTOFMEMORY;
             break;
@@ -356,19 +376,19 @@ static HRESULT create_match_array(script_ctx_t *ctx, jsstr_t *input, const match
     }
 
     while(SUCCEEDED(hres)) {
-        hres = jsdisp_propput_name(array, indexW, jsval_number(result->str-input->str));
+        hres = jsdisp_propput_name(array, indexW, jsval_number(result->cp-input-result->match_len));
         if(FAILED(hres))
             break;
 
-        hres = jsdisp_propput_name(array, lastIndexW, jsval_number(result->str-input->str+result->len));
+        hres = jsdisp_propput_name(array, lastIndexW, jsval_number(result->cp-input));
         if(FAILED(hres))
             break;
 
-        hres = jsdisp_propput_name(array, inputW, jsval_string(jsstr_addref(input)));
+        hres = jsdisp_propput_name(array, inputW, jsval_string(jsstr_addref(input_str)));
         if(FAILED(hres))
             break;
 
-        str = jsstr_alloc_len(result->str, result->len);
+        str = jsstr_alloc_len(result->cp-result->match_len, result->match_len);
         if(!str) {
             hres = E_OUTOFMEMORY;
             break;
@@ -387,13 +407,14 @@ static HRESULT create_match_array(script_ctx_t *ctx, jsstr_t *input, const match
     return S_OK;
 }
 
-static HRESULT run_exec(script_ctx_t *ctx, vdisp_t *jsthis, jsval_t arg, jsstr_t **input,
-        match_result_t *match, match_result_t **parens, DWORD *parens_cnt, BOOL *ret)
+static HRESULT run_exec(script_ctx_t *ctx, vdisp_t *jsthis, jsval_t arg,
+        jsstr_t **input, match_state_t **result, BOOL *ret)
 {
     RegExpInstance *regexp;
-    DWORD parens_size = 0, last_index = 0;
-    const WCHAR *cp;
-    jsstr_t *string;
+    match_state_t *match;
+    DWORD last_index = 0;
+    const WCHAR *string;
+    jsstr_t *jsstr;
     HRESULT hres;
 
     if(!is_vclass(jsthis, JSCLASS_REGEXP)) {
@@ -403,13 +424,13 @@ static HRESULT run_exec(script_ctx_t *ctx, vdisp_t *jsthis, jsval_t arg, jsstr_t
 
     regexp = regexp_from_vdisp(jsthis);
 
-    hres = to_string(ctx, arg, &string);
+    hres = to_flat_string(ctx, arg, &jsstr, &string);
     if(FAILED(hres))
         return hres;
 
-    if(regexp->jsregexp->flags & JSREG_GLOB) {
+    if(regexp->jsregexp->flags & REG_GLOB) {
         if(regexp->last_index < 0) {
-            jsstr_release(string);
+            jsstr_release(jsstr);
             set_last_index(regexp, 0);
             *ret = FALSE;
             if(input)
@@ -420,36 +441,43 @@ static HRESULT run_exec(script_ctx_t *ctx, vdisp_t *jsthis, jsval_t arg, jsstr_t
         last_index = regexp->last_index;
     }
 
-    cp = string->str + last_index;
-    hres = regexp_match_next(ctx, &regexp->dispex, REM_RESET_INDEX, string, &cp, parens,
-            parens ? &parens_size : NULL, parens_cnt, match);
+    match = alloc_match_state(regexp->jsregexp, &ctx->tmp_heap, string+last_index);
+    if(!match) {
+        jsstr_release(jsstr);
+        return E_OUTOFMEMORY;
+    }
+
+    hres = regexp_match_next(ctx, &regexp->dispex, REM_RESET_INDEX, jsstr, &match);
     if(FAILED(hres)) {
-        jsstr_release(string);
+        jsstr_release(jsstr);
         return hres;
     }
 
+    *result = match;
     *ret = hres == S_OK;
     if(input)
-        *input = string;
+        *input = jsstr;
     else
-        jsstr_release(string);
+        jsstr_release(jsstr);
     return S_OK;
 }
 
 static HRESULT RegExp_exec(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    match_result_t *parens = NULL, match;
-    DWORD parens_cnt = 0;
+    match_state_t *match;
+    heap_pool_t *mark;
     BOOL b;
     jsstr_t *string;
     HRESULT hres;
 
     TRACE("\n");
 
-    hres = run_exec(ctx, jsthis, argc ? argv[0] : jsval_string(jsstr_empty()), &string, &match, &parens, &parens_cnt, &b);
+    mark = heap_pool_mark(&ctx->tmp_heap);
+
+    hres = run_exec(ctx, jsthis, argc ? argv[0] : jsval_string(jsstr_empty()), &string, &match, &b);
     if(FAILED(hres)) {
-        heap_free(parens);
+        heap_pool_clear(mark);
         return hres;
     }
 
@@ -457,7 +485,7 @@ static HRESULT RegExp_exec(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsig
         if(b) {
             IDispatch *ret;
 
-            hres = create_match_array(ctx, string, &match, parens, parens_cnt, &ret);
+            hres = create_match_array(ctx, string, match, &ret);
             if(SUCCEEDED(hres))
                 *r = jsval_disp(ret);
         }else {
@@ -465,7 +493,7 @@ static HRESULT RegExp_exec(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsig
         }
     }
 
-    heap_free(parens);
+    heap_pool_clear(mark);
     jsstr_release(string);
     return hres;
 }
@@ -473,20 +501,17 @@ static HRESULT RegExp_exec(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsig
 static HRESULT RegExp_test(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
-    match_result_t match;
+    match_state_t *match;
     jsstr_t *undef_str;
+    heap_pool_t *mark;
     BOOL b;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!argc) {
-        undef_str = jsstr_alloc(undefinedW);
-        if(!undef_str)
-            return E_OUTOFMEMORY;
-    }
-
-    hres = run_exec(ctx, jsthis, argc ? argv[0] : jsval_string(undef_str), NULL, &match, NULL, NULL, &b);
+    mark = heap_pool_mark(&ctx->tmp_heap);
+    hres = run_exec(ctx, jsthis, argc ? argv[0] : jsval_string(undef_str = jsstr_undefined()), NULL, &match, &b);
+    heap_pool_clear(mark);
     if(!argc)
         jsstr_release(undef_str);
     if(FAILED(hres))
@@ -518,7 +543,7 @@ static void RegExp_destructor(jsdisp_t *dispex)
     RegExpInstance *This = (RegExpInstance*)dispex;
 
     if(This->jsregexp)
-        js_DestroyRegExp(This->jsregexp);
+        regexp_destroy(This->jsregexp);
     jsval_release(This->last_index_val);
     jsstr_release(This->str);
     heap_free(This);
@@ -587,10 +612,15 @@ static HRESULT alloc_regexp(script_ctx_t *ctx, jsdisp_t *object_prototype, RegEx
 HRESULT create_regexp(script_ctx_t *ctx, jsstr_t *src, DWORD flags, jsdisp_t **ret)
 {
     RegExpInstance *regexp;
+    const WCHAR *str;
     HRESULT hres;
 
     TRACE("%s %x\n", debugstr_jsstr(src), flags);
 
+    str = jsstr_flatten(src);
+    if(!str)
+        return E_OUTOFMEMORY;
+
     hres = alloc_regexp(ctx, NULL, &regexp);
     if(FAILED(hres))
         return hres;
@@ -598,10 +628,9 @@ HRESULT create_regexp(script_ctx_t *ctx, jsstr_t *src, DWORD flags, jsdisp_t **r
     regexp->str = jsstr_addref(src);
     regexp->last_index_val = jsval_number(0);
 
-    regexp->jsregexp = js_NewRegExp(ctx, &ctx->tmp_heap, regexp->str->str,
-            jsstr_length(regexp->str), flags, FALSE);
+    regexp->jsregexp = regexp_new(ctx, &ctx->tmp_heap, str, jsstr_length(regexp->str), flags, FALSE);
     if(!regexp->jsregexp) {
-        WARN("js_NewRegExp failed\n");
+        WARN("regexp_new failed\n");
         jsdisp_release(&regexp->dispex);
         return E_FAIL;
     }
@@ -612,8 +641,9 @@ HRESULT create_regexp(script_ctx_t *ctx, jsstr_t *src, DWORD flags, jsdisp_t **r
 
 HRESULT create_regexp_var(script_ctx_t *ctx, jsval_t src_arg, jsval_t *flags_arg, jsdisp_t **ret)
 {
-    jsstr_t *src, *opt = NULL;
-    DWORD flags;
+    unsigned flags, opt_len = 0;
+    const WCHAR *opt = NULL;
+    jsstr_t *src;
     HRESULT hres;
 
     if(is_object_instance(src_arg)) {
@@ -641,22 +671,28 @@ HRESULT create_regexp_var(script_ctx_t *ctx, jsval_t src_arg, jsval_t *flags_arg
     src = get_string(src_arg);
 
     if(flags_arg) {
+        jsstr_t *opt_str;
+
         if(!is_string(*flags_arg)) {
             FIXME("unimplemented for %s\n", debugstr_jsval(*flags_arg));
             return E_NOTIMPL;
         }
 
-        opt = get_string(*flags_arg);
+        opt_str = get_string(*flags_arg);
+        opt = jsstr_flatten(opt_str);
+        if(!opt)
+            return E_OUTOFMEMORY;
+        opt_len = jsstr_length(opt_str);
     }
 
-    hres = parse_regexp_flags(opt ? opt->str : NULL, opt ? jsstr_length(opt) : 0, &flags);
+    hres = parse_regexp_flags(opt, opt_len, &flags);
     if(FAILED(hres))
         return hres;
 
     return create_regexp(ctx, src, flags, ret);
 }
 
-HRESULT regexp_string_match(script_ctx_t *ctx, jsdisp_t *re, jsstr_t *str, jsval_t *r)
+HRESULT regexp_string_match(script_ctx_t *ctx, jsdisp_t *re, jsstr_t *jsstr, jsval_t *r)
 {
     static const WCHAR indexW[] = {'i','n','d','e','x',0};
     static const WCHAR inputW[] = {'i','n','p','u','t',0};
@@ -665,17 +701,28 @@ HRESULT regexp_string_match(script_ctx_t *ctx, jsdisp_t *re, jsstr_t *str, jsval
     RegExpInstance *regexp = (RegExpInstance*)re;
     match_result_t *match_result;
     unsigned match_cnt, i;
+    const WCHAR *str;
     jsdisp_t *array;
     HRESULT hres;
 
-    if(!(regexp->jsregexp->flags & JSREG_GLOB)) {
-        match_result_t match, *parens = NULL;
-        DWORD parens_cnt, parens_size = 0;
-        const WCHAR *cp = str->str;
+    str = jsstr_flatten(jsstr);
+    if(!str)
+        return E_OUTOFMEMORY;
 
-        hres = regexp_match_next(ctx, &regexp->dispex, 0, str, &cp, &parens, &parens_size, &parens_cnt, &match);
+    if(!(regexp->jsregexp->flags & REG_GLOB)) {
+        match_state_t *match;
+        heap_pool_t *mark;
+
+        mark = heap_pool_mark(&ctx->tmp_heap);
+        match = alloc_match_state(regexp->jsregexp, &ctx->tmp_heap, str);
+        if(!match) {
+            heap_pool_clear(mark);
+            return E_OUTOFMEMORY;
+        }
+
+        hres = regexp_match_next(ctx, &regexp->dispex, 0, jsstr, &match);
         if(FAILED(hres)) {
-            heap_free(parens);
+            heap_pool_clear(mark);
             return hres;
         }
 
@@ -683,7 +730,7 @@ HRESULT regexp_string_match(script_ctx_t *ctx, jsdisp_t *re, jsstr_t *str, jsval
             if(hres == S_OK) {
                 IDispatch *ret;
 
-                hres = create_match_array(ctx, str, &match, parens, parens_cnt, &ret);
+                hres = create_match_array(ctx, jsstr, match, &ret);
                 if(SUCCEEDED(hres))
                     *r = jsval_disp(ret);
             }else {
@@ -691,11 +738,11 @@ HRESULT regexp_string_match(script_ctx_t *ctx, jsdisp_t *re, jsstr_t *str, jsval
             }
         }
 
-        heap_free(parens);
+        heap_pool_clear(mark);
         return S_OK;
     }
 
-    hres = regexp_match(ctx, &regexp->dispex, str, FALSE, &match_result, &match_cnt);
+    hres = regexp_match(ctx, &regexp->dispex, jsstr, FALSE, &match_result, &match_cnt);
     if(FAILED(hres))
         return hres;
 
@@ -714,7 +761,7 @@ HRESULT regexp_string_match(script_ctx_t *ctx, jsdisp_t *re, jsstr_t *str, jsval
     for(i=0; i < match_cnt; i++) {
         jsstr_t *tmp_str;
 
-        tmp_str = jsstr_alloc_len(match_result[i].str, match_result[i].len);
+        tmp_str = jsstr_substr(jsstr, match_result[i].index, match_result[i].length);
         if(!tmp_str) {
             hres = E_OUTOFMEMORY;
             break;
@@ -727,16 +774,16 @@ HRESULT regexp_string_match(script_ctx_t *ctx, jsdisp_t *re, jsstr_t *str, jsval
     }
 
     while(SUCCEEDED(hres)) {
-        hres = jsdisp_propput_name(array, indexW, jsval_number(match_result[match_cnt-1].str-str->str));
+        hres = jsdisp_propput_name(array, indexW, jsval_number(match_result[match_cnt-1].index));
         if(FAILED(hres))
             break;
 
         hres = jsdisp_propput_name(array, lastIndexW,
-                jsval_number(match_result[match_cnt-1].str-str->str+match_result[match_cnt-1].len));
+                jsval_number(match_result[match_cnt-1].index + match_result[match_cnt-1].length));
         if(FAILED(hres))
             break;
 
-        hres = jsdisp_propput_name(array, inputW, jsval_string(str));
+        hres = jsdisp_propput_name(array, inputW, jsval_string(jsstr));
         break;
     }
 
@@ -755,7 +802,7 @@ static HRESULT global_idx(script_ctx_t *ctx, DWORD flags, DWORD idx, jsval_t *r)
     case DISPATCH_PROPERTYGET: {
         jsstr_t *ret;
 
-        ret = jsstr_alloc_len(ctx->match_parens[idx].str, ctx->match_parens[idx].len);
+        ret = jsstr_substr(ctx->last_match, ctx->match_parens[idx].index, ctx->match_parens[idx].length);
         if(!ret)
             return E_OUTOFMEMORY;
 
@@ -844,7 +891,7 @@ static HRESULT RegExpConstr_leftContext(script_ctx_t *ctx, vdisp_t *jsthis, WORD
     case DISPATCH_PROPERTYGET: {
         jsstr_t *ret;
 
-        ret = jsstr_alloc_len(ctx->last_match->str, ctx->last_match_index);
+        ret = jsstr_substr(ctx->last_match, 0, ctx->last_match_index);
         if(!ret)
             return E_OUTOFMEMORY;
 
@@ -870,7 +917,8 @@ static HRESULT RegExpConstr_rightContext(script_ctx_t *ctx, vdisp_t *jsthis, WOR
     case DISPATCH_PROPERTYGET: {
         jsstr_t *ret;
 
-        ret = jsstr_alloc(ctx->last_match->str+ctx->last_match_index+ctx->last_match_length);
+        ret = jsstr_substr(ctx->last_match, ctx->last_match_index+ctx->last_match_length,
+                jsstr_length(ctx->last_match) - ctx->last_match_index - ctx->last_match_length);
         if(!ret)
             return E_OUTOFMEMORY;
 
@@ -991,16 +1039,16 @@ HRESULT parse_regexp_flags(const WCHAR *str, DWORD str_len, DWORD *ret)
     for (p = str; p < str+str_len; p++) {
         switch (*p) {
         case 'g':
-            flags |= JSREG_GLOB;
+            flags |= REG_GLOB;
             break;
         case 'i':
-            flags |= JSREG_FOLD;
+            flags |= REG_FOLD;
             break;
         case 'm':
-            flags |= JSREG_MULTILINE;
+            flags |= REG_MULTILINE;
             break;
         case 'y':
-            flags |= JSREG_STICKY;
+            flags |= REG_STICKY;
             break;
         default:
             WARN("wrong flag %c\n", *p);
