@@ -30,6 +30,30 @@
 
 #include "wine/test.h"
 
+#ifdef _WIN64
+
+#define IActiveScriptParse_QueryInterface IActiveScriptParse64_QueryInterface
+#define IActiveScriptParse_Release IActiveScriptParse64_Release
+#define IActiveScriptParse_InitNew IActiveScriptParse64_InitNew
+#define IActiveScriptParse_ParseScriptText IActiveScriptParse64_ParseScriptText
+#define IActiveScriptParseProcedure2_Release \
+    IActiveScriptParseProcedure2_64_Release
+#define IActiveScriptParseProcedure2_ParseProcedureText \
+    IActiveScriptParseProcedure2_64_ParseProcedureText
+
+#else
+
+#define IActiveScriptParse_QueryInterface IActiveScriptParse32_QueryInterface
+#define IActiveScriptParse_Release IActiveScriptParse32_Release
+#define IActiveScriptParse_InitNew IActiveScriptParse32_InitNew
+#define IActiveScriptParse_ParseScriptText IActiveScriptParse32_ParseScriptText
+#define IActiveScriptParseProcedure2_Release \
+    IActiveScriptParseProcedure2_32_Release
+#define IActiveScriptParseProcedure2_ParseProcedureText \
+    IActiveScriptParseProcedure2_32_ParseProcedureText
+
+#endif
+
 static const CLSID CLSID_JScript =
     {0xf414c260,0x6ac0,0x11cf,{0xb6,0xd1,0x00,0xaa,0x00,0xbb,0xbb,0x58}};
 
@@ -696,7 +720,7 @@ static void _parse_script_a(unsigned line, IActiveScriptParse *parser, const cha
     HRESULT hres;
 
     str = a2bstr(script);
-    hres = IActiveScriptParse64_ParseScriptText(parser, str, NULL, NULL, NULL, 0, 0, 0, NULL, NULL);
+    hres = IActiveScriptParse_ParseScriptText(parser, str, NULL, NULL, NULL, 0, 0, 0, NULL, NULL);
     SysFreeString(str);
     ok_(__FILE__,line)(hres == S_OK, "ParseScriptText failed: %08x\n", hres);
 }
@@ -731,7 +755,7 @@ static IActiveScriptParse *create_script(BOOL skip_tests, BOOL use_sec_mgr)
     hres = IActiveScript_QueryInterface(script, &IID_IActiveScriptParse, (void**)&parser);
     ok(hres == S_OK, "Could not get IActiveScriptParse: %08x\n", hres);
 
-    hres = IActiveScriptParse64_InitNew(parser);
+    hres = IActiveScriptParse_InitNew(parser);
     ok(hres == S_OK, "InitNew failed: %08x\n", hres);
 
     hres = IActiveScript_SetScriptSite(script, &ActiveScriptSite);
@@ -771,18 +795,18 @@ static IDispatchEx *parse_procedure_a(IActiveScriptParse *parser, const char *sr
     BSTR str;
     HRESULT hres;
 
-    hres = IUnknown_QueryInterface(parser, &IID_IActiveScriptParseProcedure2, (void**)&parse_proc);
+    hres = IActiveScriptParse_QueryInterface(parser, &IID_IActiveScriptParseProcedure2, (void**)&parse_proc);
     ok(hres == S_OK, "Could not get IActiveScriptParseProcedure2: %08x\n", hres);
 
     str = a2bstr(src);
-    hres = IActiveScriptParseProcedure2_64_ParseProcedureText(parse_proc, str, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, &disp);
+    hres = IActiveScriptParseProcedure2_ParseProcedureText(parse_proc, str, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, &disp);
     SysFreeString(str);
-    IUnknown_Release(parse_proc);
+    IActiveScriptParseProcedure2_Release(parse_proc);
     ok(hres == S_OK, "ParseProcedureText failed: %08x\n", hres);
     ok(disp != NULL, "disp == NULL\n");
 
     hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
-    IDispatch_Release(dispex);
+    IDispatch_Release(disp);
     ok(hres == S_OK, "Could not get IDispatchEx iface: %08x\n", hres);
 
     return dispex;
@@ -848,7 +872,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(reportSuccess);
 
     IDispatchEx_Release(proc);
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     proc = parse_procedure_a(parser, "(new ActiveXObject('Wine.Test')).reportSuccess();");
@@ -870,7 +894,7 @@ static void test_ActiveXObject(void)
     parse_script_a(parser, "testException(function() { new ActiveXObject('Wine.TestABC'); }, 'Error', -2146827859);");
 
     IDispatchEx_Release(proc);
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     QS_SecMgr_hres = E_NOINTERFACE;
@@ -879,7 +903,7 @@ static void test_ActiveXObject(void)
     parse_script_a(parser, "testException(function() { new ActiveXObject('Wine.Test'); }, 'Error', -2146827859);");
     CHECK_CALLED(Host_QS_SecMgr);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     ProcessUrlAction_hres = E_FAIL;
@@ -890,7 +914,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(Host_QS_SecMgr);
     CHECK_CALLED(ProcessUrlAction);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     ProcessUrlAction_policy = URLPOLICY_DISALLOW;
@@ -901,7 +925,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(Host_QS_SecMgr);
     CHECK_CALLED(ProcessUrlAction);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     CreateInstance_hres = E_FAIL;
@@ -914,7 +938,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(ProcessUrlAction);
     CHECK_CALLED(CreateInstance);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     QueryCustomPolicy_hres = E_FAIL;
@@ -929,7 +953,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(CreateInstance);
     CHECK_CALLED(QueryCustomPolicy);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     QueryCustomPolicy_psize = 6;
@@ -948,7 +972,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(QI_IObjectWithSite);
     CHECK_CALLED(reportSuccess);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     QueryCustomPolicy_policy = URLPOLICY_DISALLOW;
@@ -984,7 +1008,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(CreateInstance);
     CHECK_CALLED(QueryCustomPolicy);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, FALSE);
 
@@ -996,7 +1020,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(QI_IObjectWithSite);
     CHECK_CALLED(reportSuccess);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     object_with_site = &ObjectWithSite;
@@ -1030,7 +1054,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(QI_IObjectWithSite);
     CHECK_CALLED(SetSite);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     /* No IServiceProvider Interface */
     parser = create_script(FALSE, FALSE);
@@ -1047,7 +1071,7 @@ static void test_ActiveXObject(void)
     CHECK_CALLED(reportSuccess);
     CHECK_CALLED(SetSite);
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 
     parser = create_script(FALSE, TRUE);
     object_with_site = &ObjectWithSite;
@@ -1055,7 +1079,7 @@ static void test_ActiveXObject(void)
 
     parse_script_a(parser, "testException(function() { new ActiveXObject('Wine.Test'); }, 'Error', -2146827859);");
 
-    IUnknown_Release(parser);
+    IActiveScriptParse_Release(parser);
 }
 
 static BOOL init_key(const char *key_name, const char *def_value, BOOL init)
@@ -1114,12 +1138,12 @@ static BOOL check_jscript(void)
         return FALSE;
 
     str = a2bstr("if(!('localeCompare' in String.prototype)) throw 1;");
-    hres = IActiveScriptParse64_ParseScriptText(parser, str, NULL, NULL, NULL, 0, 0, 0, NULL, NULL);
+    hres = IActiveScriptParse_ParseScriptText(parser, str, NULL, NULL, NULL, 0, 0, 0, NULL, NULL);
     SysFreeString(str);
 
     if(hres == S_OK)
-        hres = IUnknown_QueryInterface(parser, &IID_IActiveScriptProperty, (void**)&script_prop);
-    IUnknown_Release(parser);
+        hres = IActiveScriptParse_QueryInterface(parser, &IID_IActiveScriptProperty, (void**)&script_prop);
+    IActiveScriptParse_Release(parser);
     if(hres == S_OK)
         IActiveScriptProperty_Release(script_prop);
 
