@@ -492,7 +492,7 @@ char* CDECL _Getmonths(void)
 void* CDECL _Gettnames(void)
 {
     MSVCRT___lc_time_data *ret, *cur = get_locinfo()->lc_time_curr;
-    int i, size = sizeof(MSVCRT___lc_time_data);
+    unsigned int i, size = sizeof(MSVCRT___lc_time_data);
 
     TRACE("\n");
 
@@ -702,7 +702,9 @@ void free_mbcinfo(MSVCRT_pthreadmbcinfo mbcinfo)
     MSVCRT_free(mbcinfo);
 }
 
-/* _get_current_locale - not exported in native msvcrt */
+/*********************************************************************
+ *      _get_current_locale (MSVCRT.@)
+ */
 MSVCRT__locale_t CDECL MSVCRT__get_current_locale(void)
 {
     MSVCRT__locale_t loc = MSVCRT_malloc(sizeof(MSVCRT__locale_tstruct));
@@ -716,7 +718,9 @@ MSVCRT__locale_t CDECL MSVCRT__get_current_locale(void)
     return loc;
 }
 
-/* _free_locale - not exported in native msvcrt */
+/*********************************************************************
+ *      _free_locale (MSVCRT.@)
+ */
 void CDECL MSVCRT__free_locale(MSVCRT__locale_t locale)
 {
     if (!locale)
@@ -727,7 +731,9 @@ void CDECL MSVCRT__free_locale(MSVCRT__locale_t locale)
     MSVCRT_free(locale);
 }
 
-/* _create_locale - not exported in native msvcrt */
+/*********************************************************************
+ *      _create_locale (MSVCRT.@)
+ */
 MSVCRT__locale_t CDECL MSVCRT__create_locale(int category, const char *locale)
 {
     static const DWORD time_data[] = {
@@ -1216,7 +1222,7 @@ MSVCRT__locale_t CDECL MSVCRT__create_locale(int category, const char *locale)
     for(i=0; i<sizeof(time_data)/sizeof(time_data[0]); i++) {
         if(time_data[i]==LOCALE_SSHORTDATE && !lcid[MSVCRT_LC_TIME]) {
             size += sizeof(cloc_short_date) + sizeof(cloc_short_dateW);
-        }else if(time_data[i]==LOCALE_SSHORTDATE && !lcid[MSVCRT_LC_TIME]) {
+        }else if(time_data[i]==LOCALE_SLONGDATE && !lcid[MSVCRT_LC_TIME]) {
             size += sizeof(cloc_long_date) + sizeof(cloc_long_dateW);
         }else {
             ret = GetLocaleInfoA(lcid_tmp, time_data[i]
@@ -1249,7 +1255,7 @@ MSVCRT__locale_t CDECL MSVCRT__create_locale(int category, const char *locale)
         if(time_data[i]==LOCALE_SSHORTDATE && !lcid[MSVCRT_LC_TIME]) {
             memcpy(&loc->locinfo->lc_time_curr->data[ret], cloc_short_date, sizeof(cloc_short_date));
             ret += sizeof(cloc_short_date);
-        }else if(time_data[i]==LOCALE_SSHORTDATE && !lcid[MSVCRT_LC_TIME]) {
+        }else if(time_data[i]==LOCALE_SLONGDATE && !lcid[MSVCRT_LC_TIME]) {
             memcpy(&loc->locinfo->lc_time_curr->data[ret], cloc_long_date, sizeof(cloc_long_date));
             ret += sizeof(cloc_long_date);
         }else if(time_data[i]==LOCALE_STIMEFORMAT && !lcid[MSVCRT_LC_TIME]) {
@@ -1265,7 +1271,7 @@ MSVCRT__locale_t CDECL MSVCRT__create_locale(int category, const char *locale)
         if(time_data[i]==LOCALE_SSHORTDATE && !lcid[MSVCRT_LC_TIME]) {
             memcpy(&loc->locinfo->lc_time_curr->data[ret], cloc_short_dateW, sizeof(cloc_short_dateW));
             ret += sizeof(cloc_short_dateW);
-        }else if(time_data[i]==LOCALE_SSHORTDATE && !lcid[MSVCRT_LC_TIME]) {
+        }else if(time_data[i]==LOCALE_SLONGDATE && !lcid[MSVCRT_LC_TIME]) {
             memcpy(&loc->locinfo->lc_time_curr->data[ret], cloc_long_dateW, sizeof(cloc_long_dateW));
             ret += sizeof(cloc_long_dateW);
         }else if(time_data[i]==LOCALE_STIMEFORMAT && !lcid[MSVCRT_LC_TIME]) {
