@@ -571,6 +571,19 @@ int CDECL MSVCRT__strnicoll( const char* str1, const char* str2, MSVCRT_size_t c
     return MSVCRT__strnicoll_l(str1, str2, count, NULL);
 }
 
+/*********************************************************************
+ *                  strncpy (MSVCRT.@)
+ */
+char* __cdecl MSVCRT_strncpy(char *dst, const char *src, MSVCRT_size_t len)
+{
+    MSVCRT_size_t i;
+
+    for(i=0; i<len; i++)
+        if((dst[i] = src[i]) == '\0') break;
+
+    return dst;
+}
+
 /*********************************************************************
  *      strcpy_s (MSVCRT.@)
  */
@@ -861,7 +874,7 @@ __int64 CDECL MSVCRT_strtoi64(const char *nptr, char **endptr, int base)
 /*********************************************************************
  *  _atoi_l (MSVCRT.@)
  */
-int MSVCRT__atoi_l(const char *str, MSVCRT__locale_t locale)
+int __cdecl MSVCRT__atoi_l(const char *str, MSVCRT__locale_t locale)
 {
     __int64 ret = MSVCRT_strtoi64_l(str, NULL, 10, locale);
 
@@ -875,6 +888,14 @@ int MSVCRT__atoi_l(const char *str, MSVCRT__locale_t locale)
     return ret;
 }
 
+/*********************************************************************
+ *  atoi (MSVCRT.@)
+ */
+int __cdecl MSVCRT_atoi(const char *str)
+{
+    return MSVCRT__atoi_l(str, NULL);
+}
+
 /*********************************************************************
  *  _strtoui64_l (MSVCRT.@)
  *
@@ -1187,7 +1208,7 @@ int CDECL MSVCRT__ui64tow_s( unsigned __int64 value, MSVCRT_wchar_t *str,
         return MSVCRT_EINVAL;
     }
 
-    memcpy(str, pos, buffer-pos+65);
+    memcpy(str, pos, (buffer-pos+65)*sizeof(MSVCRT_wchar_t));
     return 0;
 }
 
@@ -1248,7 +1269,7 @@ int CDECL _ultoa_s(MSVCRT_ulong value, char *str, MSVCRT_size_t size, int radix)
 /*********************************************************************
  *  _ultow_s (MSVCRT.@)
  */
-int CDECL _ultow_s(MSVCRT_ulong value, WCHAR *str, MSVCRT_size_t size, int radix)
+int CDECL _ultow_s(MSVCRT_ulong value, MSVCRT_wchar_t *str, MSVCRT_size_t size, int radix)
 {
     MSVCRT_ulong digit;
     WCHAR buffer[33], *pos;
@@ -1295,7 +1316,7 @@ int CDECL _ultow_s(MSVCRT_ulong value, WCHAR *str, MSVCRT_size_t size, int radix
         return MSVCRT_ERANGE;
     }
 
-    memcpy(str, pos, len * sizeof(WCHAR));
+    memcpy(str, pos, len * sizeof(MSVCRT_wchar_t));
     return 0;
 }
 
@@ -1558,6 +1579,30 @@ void * __cdecl MSVCRT_memcpy( void *dst, const void *src, size_t n )
     return memmove( dst, src, n );
 }
 
+/*********************************************************************
+ *		    memset (MSVCRT.@)
+ */
+void* __cdecl MSVCRT_memset(void *dst, int c, MSVCRT_size_t n)
+{
+    return memset(dst, c, n);
+}
+
+/*********************************************************************
+ *		    strchr (MSVCRT.@)
+ */
+char* __cdecl MSVCRT_strchr(const char *str, int c)
+{
+    return strchr(str, c);
+}
+
+/*********************************************************************
+ *                  strncmp   (MSVCRT.@)
+ */
+int __cdecl MSVCRT_strncmp(const char *str1, const char *str2, MSVCRT_size_t len)
+{
+    return strncmp(str1, str2, len);
+}
+
 /*********************************************************************
  *                  _strnicmp_l   (MSVCRT.@)
  */
