@@ -394,7 +394,6 @@ HIC VFWAPI ICOpen(DWORD fccType, DWORD fccHandler, UINT wMode)
     ICOPEN		icopen;
     HDRVR		hdrv;
     WINE_HIC*           whic;
-    BOOL                bIs16;
     static const WCHAR  drv32W[] = {'d','r','i','v','e','r','s','3','2','\0'};
     reg_driver*         driver;
 
@@ -404,9 +403,11 @@ HIC VFWAPI ICOpen(DWORD fccType, DWORD fccHandler, UINT wMode)
     driver = reg_driver_list;
     while(driver)
         if (!compare_fourcc(fccType, driver->fccType) &&
-            !compare_fourcc(fccHandler, driver->fccHandler))
+            !compare_fourcc(fccHandler, driver->fccHandler)) {
+	    fccType = driver->fccType;
+	    fccHandler = driver->fccHandler;
 	    break;
-        else
+        } else
             driver = driver->next;
 
     if (driver && driver->proc)
@@ -427,7 +428,13 @@ HIC VFWAPI ICOpen(DWORD fccType, DWORD fccHandler, UINT wMode)
     icopen.dnDevNode            = 0; /* FIXME */
 	
     if (!driver) {
-        /* The driver is registered in the registry */
+        /* normalize to lower case as in 'vidc' */
+        ((char*)&fccType)[0] = tolower(((char*)&fccType)[0]);
+        ((char*)&fccType)[1] = tolower(((char*)&fccType)[1]);
+        ((char*)&fccType)[2] = tolower(((char*)&fccType)[2]);
+        ((char*)&fccType)[3] = tolower(((char*)&fccType)[3]);
+        icopen.fccType = fccType;
+        /* Seek the driver in the registry */
 	fourcc_to_string(codecname, fccType);
         codecname[4] = '.';
 	fourcc_to_string(codecname + 5, fccHandler);
@@ -442,14 +449,7 @@ HIC VFWAPI ICOpen(DWORD fccType, DWORD fccHandler, UINT wMode)
         if (!hdrv) 
             return 0; 
     }
-    bIs16 = GetDriverFlags(hdrv) & 0x10000000; /* undocumented flag: WINE_GDF_16BIT */
 
-    if (bIs16)
-    {
-        FIXME("Got a 16 bit driver, but no 16 bit support in msvfw\n");
-        CloseDriver(hdrv, 0, 0);
-        return 0;
-    }
     whic = HeapAlloc(GetProcessHeap(), 0, sizeof(WINE_HIC));
     if (!whic)
     {
@@ -472,7 +472,7 @@ HIC VFWAPI ICOpen(DWORD fccType, DWORD fccHandler, UINT wMode)
 /***********************************************************************
  *		ICOpenFunction			[MSVFW32.@]
  */
-HIC VFWAPI ICOpenFunction(DWORD fccType, DWORD fccHandler, UINT wMode, FARPROC lpfnHandler)
+HIC VFWAPI ICOpenFunction(DWORD fccType, DWORD fccHandler, UINT wMode, DRIVERPROC lpfnHandler)
 {
     ICOPEN      icopen;
     WINE_HIC*   whic;
@@ -538,7 +538,6 @@ LRESULT VFWAPI ICGetInfo(HIC hic, ICINFO *picinfo, DWORD cb)
 
     TRACE("(%p,%p,%d)\n", hic, picinfo, cb);
 
-    whic = MSVIDEO_GetHicPtr(hic);
     if (!whic) return ICERR_BADHANDLE;
     if (!picinfo) return MMSYSERR_INVALPARAM;
 
@@ -1513,7 +1512,10 @@ static BOOL GetFileNamePreview(LPVOID lpofn,BOOL bSave,BOOL bUnicode)
 
   fnGetFileName = (LPVOID)GetProcAddress(hComdlg32, szFunctionName);
   if (fnGetFileName == NULL)
+  {
+    FreeLibrary(hComdlg32);
     return FALSE;
+  }
 
   /* FIXME: need to add OFN_ENABLEHOOK and our own handler */
   ret = fnGetFileName(lpofn);
