@@ -91,7 +91,7 @@ static HRESULT WINAPI domattr_QueryInterface(
         return E_NOINTERFACE;
     }
 
-    IXMLDOMText_AddRef((IUnknown*)*ppvObject);
+    IXMLDOMAttribute_AddRef(iface);
     return S_OK;
 }
 
@@ -603,7 +603,7 @@ static HRESULT WINAPI domattr_put_value(
 
     TRACE("(%p)->(%s)\n", This, debugstr_variant(&value));
 
-    return node_put_value(&This->node, &value);
+    return node_put_value_escaped(&This->node, &value);
 }
 
 static const struct IXMLDOMAttributeVtbl domattr_vtbl =
