@@ -357,6 +357,7 @@ __ASM_GLOBAL_FUNC( call_entry_point,
                    "ret" )
 #elif defined(__arm__)
 __ASM_GLOBAL_FUNC( call_entry_point,
+                   ".arm\n\t"
                    "push {r4, r5, LR}\n\t"
                    "mov r4, r0\n\t"
                    "mov r5, SP\n\t"
@@ -364,7 +365,8 @@ __ASM_GLOBAL_FUNC( call_entry_point,
                    "cmp r3, #0\n\t"
                    "beq 5f\n\t"
                    "sub SP, SP, r3\n\t"
-                   "bic SP, SP, #15\n\t"
+                   "tst r1, #1\n\t"
+                   "subeq SP, SP, #4\n\t"
                    "1:\tsub r3, r3, #4\n\t"
                    "ldr r0, [r2, r3]\n\t"
                    "str r0, [SP, r3]\n\t"
