@@ -145,6 +145,8 @@ static int get_length(DBTYPE type)
     case DBTYPE_STR:
     case DBTYPE_BYREF | DBTYPE_WSTR:
         return 0;
+    case DBTYPE_VARIANT:
+        return sizeof(VARIANT);
     default:
         FIXME("Unhandled type %04x\n", type);
         return 0;
@@ -161,12 +163,18 @@ static HRESULT WINAPI convert_DataConvert(IDataConvert* iface,
                                           DBDATACONVERT flags)
 {
     convert *This = impl_from_IDataConvert(iface);
+    DBLENGTH dst_len_loc;
+    DBSTATUS dst_status_loc;
+    VARIANT tmp;
     HRESULT hr;
 
     TRACE("(%p)->(%d, %d, %ld, %p, %p, %p, %ld, %d, %p, %d, %d, %x)\n", This,
           src_type, dst_type, src_len, dst_len, src, dst, dst_max_len,
           src_status, dst_status, precision, scale, flags);
 
+    if (!dst_len) dst_len = &dst_len_loc;
+    if (!dst_status) dst_status = &dst_status_loc;
+
     *dst_status = DBSTATUS_E_BADACCESSOR;
 
     if(IDataConvert_CanConvert(iface, src_type, dst_type) != S_OK)
@@ -219,7 +227,6 @@ static HRESULT WINAPI convert_DataConvert(IDataConvert* iface,
     case DBTYPE_I2:
     {
         signed short *d = dst;
-        VARIANT tmp;
         switch(src_type)
         {
         case DBTYPE_EMPTY:       *d = 0; hr = S_OK;                              break;
@@ -251,7 +258,6 @@ static HRESULT WINAPI convert_DataConvert(IDataConvert* iface,
     case DBTYPE_I4:
     {
         signed int *d = dst;
-        VARIANT tmp;
         switch(src_type)
         {
         case DBTYPE_EMPTY:       *d = 0; hr = S_OK;                              break;
@@ -280,6 +286,18 @@ static HRESULT WINAPI convert_DataConvert(IDataConvert* iface,
         break;
     }
 
+    case DBTYPE_I8:
+    {
+        LONGLONG *d = dst;
+        switch(src_type)
+        {
+        case DBTYPE_BSTR:        hr = VarI8FromStr(*(WCHAR**)src, LOCALE_USER_DEFAULT, 0, d); break;
+        case DBTYPE_I8:          *d = *(LONGLONG*)src; hr = S_OK;              break;
+        default: FIXME("Unimplemented conversion %04x -> I8\n", src_type); return E_NOTIMPL;
+        }
+        break;
+    }
+
     case DBTYPE_R4:
     {
         FLOAT *d = dst;
@@ -503,6 +521,11 @@ static HRESULT WINAPI convert_DataConvert(IDataConvert* iface,
             }
         }
         break;
+        case DBTYPE_VARIANT:
+            VariantInit(&tmp);
+            if ((hr = VariantChangeType(&tmp, (VARIANT*)src, 0, VT_BSTR)) == S_OK)
+                *d = V_BSTR(&tmp);
+            break;
         default: FIXME("Unimplemented conversion %04x -> BSTR\n", src_type); return E_NOTIMPL;
         }
         break;
@@ -580,6 +603,11 @@ static HRESULT WINAPI convert_DataConvert(IDataConvert* iface,
         case DBTYPE_UI4:         *d = *(DWORD*)src; hr = S_OK;                   break;
         case DBTYPE_I8:          hr = VarUI4FromI8(*(LONGLONG*)src, d);          break;
         case DBTYPE_UI8:         hr = VarUI4FromUI8(*(ULONGLONG*)src, d);        break;
+        case DBTYPE_VARIANT:
+            VariantInit(&tmp);
+            if ((hr = VariantChangeType(&tmp, (VARIANT*)src, 0, VT_UI4)) == S_OK)
+                *d = V_UI4(&tmp);
+            break;
         default: FIXME("Unimplemented conversion %04x -> UI4\n", src_type); return E_NOTIMPL;
         }
         break;
@@ -718,10 +746,58 @@ static HRESULT WINAPI convert_DataConvert(IDataConvert* iface,
         return hr;
     }
 
+    case DBTYPE_VARIANT:
+    {
+        VARIANT *v = dst;
+
+        switch(src_type)
+        {
+        case DBTYPE_BOOL:
+            V_VT(v) = VT_BOOL;
+            V_BOOL(v) = *(VARIANT_BOOL*)src;
+            hr = S_OK;
+            break;
+        case DBTYPE_I4:
+            V_VT(v) = VT_I4;
+            V_I4(v) = *(signed int*)src;
+            hr = S_OK;
+            break;
+        case DBTYPE_I8:
+            V_VT(v) = VT_DECIMAL;
+            hr = VarDecFromI8( *(LONGLONG*)src,  &V_DECIMAL(v));
+            break;
+        case DBTYPE_R8:
+            V_VT(v) = VT_R8;
+            V_R8(v) = *(double*)src;
+            hr = S_OK;
+            break;
+        case DBTYPE_BSTR:
+        {
+            BSTR s = *(WCHAR**)src;
+            TRACE("%s\n", debugstr_w(s));
+            V_VT(v) = VT_BSTR;
+            V_BSTR(v) = SysAllocString(s);
+            hr = V_BSTR(v) ? S_OK : E_OUTOFMEMORY;
+            break;
+        }
+        case DBTYPE_DATE:
+            V_VT(v) = VT_DATE;
+            V_DATE(v) = *(DATE*)src;
+            hr = S_OK;
+            break;
+        case DBTYPE_CY:
+            V_VT(v) = VT_CY;
+            V_CY(v) = *(CY*)src;
+            hr = S_OK;
+            break;
+        default: FIXME("Unimplemented conversion %04x -> VARIANT\n", src_type); return E_NOTIMPL;
+        }
+        break;
+    }
+
     default:
         FIXME("Unimplemented conversion %04x -> %04x\n", src_type, dst_type);
         return E_NOTIMPL;
-
     }
 
     if(hr == DISP_E_OVERFLOW)
@@ -987,14 +1063,80 @@ static HRESULT WINAPI convert_CanConvert(IDataConvert* iface,
 }
 
 static HRESULT WINAPI convert_GetConversionSize(IDataConvert* iface,
-                                                DBTYPE wSrcType, DBTYPE wDstType,
-                                                DBLENGTH *pcbSrcLength, DBLENGTH *pcbDstLength,
-                                                void *pSrc)
+                                                DBTYPE src_type, DBTYPE dst_type,
+                                                DBLENGTH *src_len, DBLENGTH *dst_len,
+                                                void *src)
 {
     convert *This = impl_from_IDataConvert(iface);
-    FIXME("(%p)->(%d, %d, %p, %p, %p): stub\n", This, wSrcType, wDstType, pcbSrcLength, pcbDstLength, pSrc);
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("(%p)->(%d, %d, %p, %p, %p)\n", This, src_type, dst_type, src_len, dst_len, src);
+
+    hr = IDataConvert_CanConvert(iface, src_type, dst_type);
+    if (hr != S_OK)
+        return DB_E_UNSUPPORTEDCONVERSION;
+
+    if (!dst_len)
+        return E_INVALIDARG;
+
+    /* for some types we don't need to look into source data */
+    if ((*dst_len = get_length(dst_type)))
+        return S_OK;
+
+    switch (dst_type)
+    {
+    case DBTYPE_STR:
+    {
+        switch (src_type)
+        {
+        case DBTYPE_VARIANT:
+        {
+            VARIANT v;
+
+            VariantInit(&v);
+            if ((hr = VariantChangeType(&v, (VARIANT*)src, 0, VT_BSTR)) == S_OK)
+            {
+                *dst_len = WideCharToMultiByte(CP_ACP, 0, V_BSTR(&v), -1, NULL, 0, NULL, NULL);
+                VariantClear(&v);
+            }
+            else
+                return hr;
+        }
+        break;
+        default:
+            FIXME("unimplemented for %04x -> DBTYPE_STR\n", src_type);
+            return E_NOTIMPL;
+        }
+    }
+    break;
+    case DBTYPE_WSTR:
+    {
+        switch (src_type)
+        {
+        case DBTYPE_STR:
+            if(src_len)
+                *dst_len = (*src_len + 1) * sizeof(WCHAR);
+            else
+                *dst_len = (strlen(src) + 1) * sizeof(WCHAR);
+            break;
+        case DBTYPE_WSTR:
+            if(src_len)
+                *dst_len = (*src_len) + sizeof(WCHAR);
+            else
+                *dst_len = (lstrlenW(src) + 1) * sizeof(WCHAR);
+            break;
+        default:
+            FIXME("unimplemented for %04x -> DBTYPE_WSTR\n", src_type);
+            return E_NOTIMPL;
+        }
+    }
+    break;
+    default:
+        FIXME("unimplemented for conversion %d->%d\n", src_type, dst_type);
+        return E_NOTIMPL;
+    }
+
+    return S_OK;
 }
 
 static const struct IDataConvertVtbl convert_vtbl =
