@@ -133,6 +133,15 @@ DWORD WINAPI lineConfigDialogA(DWORD dwDeviceID, HWND hwndOwner, LPCSTR lpszDevi
     return 0;
 }
 
+/***********************************************************************
+ *		lineConfigDialogW (TAPI32.@)
+ */
+DWORD WINAPI lineConfigDialogW(DWORD dwDeviceID, HWND hwndOwner, LPCWSTR lpszDeviceClass)
+{
+    FIXME("(%08x, %p, %s): stub.\n", dwDeviceID, hwndOwner, debugstr_w(lpszDeviceClass));
+    return 0;
+}
+
 /***********************************************************************
  *		lineConfigDialogEdit (TAPI32.@)
  */
@@ -187,6 +196,15 @@ DWORD WINAPI lineDialA(HCALL hCall, LPCSTR lpszDestAddress, DWORD dwCountryCode)
     return 1;
 }
 
+/***********************************************************************
+ *		lineDialW (TAPI32.@)
+ */
+DWORD WINAPI lineDialW(HCALL hCall, LPCWSTR lpszDestAddress, DWORD dwCountryCode)
+{
+    FIXME("(%p, %s, %08x): stub.\n", hCall, debugstr_w(lpszDestAddress), dwCountryCode);
+    return 1;
+}
+
 /***********************************************************************
  *		lineDrop (TAPI32.@)
  */
@@ -465,6 +483,14 @@ DWORD WINAPI lineGetCountryA(DWORD dwCountryID, DWORD dwAPIVersion, LPLINECOUNTR
     return 0;
 }
 
+/***********************************************************************
+ *		lineGetCountry (TAPI32.@)
+ */
+DWORD WINAPI lineGetCountryW(DWORD dwCountryID, DWORD dwAPIVersion, LPLINECOUNTRYLIST lpLineCountryList)
+{
+    return lineGetCountryA(dwCountryID, dwAPIVersion, lpLineCountryList);
+}
+
 /***********************************************************************
  *		lineGetDevCapsW (TAPI32.@)
  */
@@ -1389,6 +1415,18 @@ DWORD WINAPI lineTranslateAddressA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dw
     return 0;
 }
 
+/***********************************************************************
+ *              lineTranslateAddressW (TAPI32.@)
+ */
+DWORD WINAPI lineTranslateAddressW(HLINEAPP hLineApp, DWORD dwDeviceID,
+        DWORD dwAPIVersion, LPCWSTR lpszAddressIn, DWORD dwCard,
+        DWORD dwTranslateOptions, LPLINETRANSLATEOUTPUT lpTranslateOutput)
+{
+    FIXME("(%p, %08x, %08x, %s, %08x, %08x, %p): stub.\n", hLineApp, dwDeviceID, dwAPIVersion,
+            debugstr_w(lpszAddressIn), dwCard, dwTranslateOptions, lpTranslateOutput);
+    return 0;
+}
+
 /***********************************************************************
  *		lineTranslateDialog (TAPI32.@)
  */
@@ -1398,6 +1436,17 @@ DWORD WINAPI lineTranslateDialogA(HLINEAPP hLineApp, DWORD dwDeviceID, DWORD dwA
     return 0;
 }
 
+/***********************************************************************
+ *              lineTranslateDialogW (TAPI32.@)
+ */
+DWORD WINAPI lineTranslateDialogW(HLINEAPP hLineApp, DWORD dwDeviceID,
+        DWORD dwAPIVersion, HWND hwndOwner, LPCWSTR lpszAddressIn)
+{
+    FIXME("(%p, %08x, %08x, %p, %s): stub.\n", hLineApp, dwDeviceID,
+            dwAPIVersion, hwndOwner, debugstr_w(lpszAddressIn));
+    return 0;
+}
+
 /***********************************************************************
  *		lineUncompleteCall (TAPI32.@)
  */
