@@ -76,7 +76,7 @@ static HRESULT read_png_chunk(IStream *stream, BYTE *type, BYTE **data, ULONG *d
             return hr;
         }
 
-        /* FIXME: Verify the CRC? */
+        /* Windows ignores CRC of the chunk */
     }
 
     return S_OK;
@@ -174,6 +174,7 @@ MAKE_FUNCPTR(png_get_pHYs);
 MAKE_FUNCPTR(png_get_PLTE);
 MAKE_FUNCPTR(png_get_tRNS);
 MAKE_FUNCPTR(png_set_bgr);
+MAKE_FUNCPTR(png_set_crc_action);
 MAKE_FUNCPTR(png_set_error_fn);
 #if HAVE_PNG_SET_EXPAND_GRAY_1_2_4_TO_8
 MAKE_FUNCPTR(png_set_expand_gray_1_2_4_to_8);
@@ -222,6 +223,7 @@ static void *load_libpng(void)
         LOAD_FUNCPTR(png_get_PLTE);
         LOAD_FUNCPTR(png_get_tRNS);
         LOAD_FUNCPTR(png_set_bgr);
+        LOAD_FUNCPTR(png_set_crc_action);
         LOAD_FUNCPTR(png_set_error_fn);
 #if HAVE_PNG_SET_EXPAND_GRAY_1_2_4_TO_8
         LOAD_FUNCPTR(png_set_expand_gray_1_2_4_to_8);
@@ -438,6 +440,7 @@ static HRESULT WINAPI PngDecoder_Initialize(IWICBitmapDecoder *iface, IStream *p
         goto end;
     }
     ppng_set_error_fn(This->png_ptr, jmpbuf, user_error_fn, user_warning_fn);
+    ppng_set_crc_action(This->png_ptr, PNG_CRC_QUIET_USE, PNG_CRC_QUIET_USE);
 
     /* seek to the start of the stream */
     seek.QuadPart = 0;
@@ -795,7 +798,7 @@ static HRESULT WINAPI PngDecoder_Frame_CopyPalette(IWICBitmapFrameDecode *iface,
     png_colorp png_palette;
     int num_palette;
     WICColor palette[256];
-    png_bytep trans;
+    png_bytep trans_alpha;
     int num_trans;
     png_color_16p trans_values;
     int i;
@@ -819,23 +822,18 @@ static HRESULT WINAPI PngDecoder_Frame_CopyPalette(IWICBitmapFrameDecode *iface,
         goto end;
     }
 
+    ret = ppng_get_tRNS(This->png_ptr, This->info_ptr, &trans_alpha, &num_trans, &trans_values);
+    if (!ret) num_trans = 0;
+
     for (i=0; i<num_palette; i++)
     {
-        palette[i] = (0xff000000|
+        BYTE alpha = (i < num_trans) ? trans_alpha[i] : 0xff;
+        palette[i] = (alpha << 24 |
                       png_palette[i].red << 16|
                       png_palette[i].green << 8|
                       png_palette[i].blue);
     }
 
-    ret = ppng_get_tRNS(This->png_ptr, This->info_ptr, &trans, &num_trans, &trans_values);
-    if (ret)
-    {
-        for (i=0; i<num_trans; i++)
-        {
-            palette[trans[i]] = 0x00000000;
-        }
-    }
-
 end:
 
     LeaveCriticalSection(&This->lock);
