@@ -252,10 +252,7 @@ HRESULT CDECL wined3d_vertex_declaration_create(struct wined3d_device *device,
 
     object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
     if(!object)
-    {
-        ERR("Failed to allocate vertex declaration memory.\n");
         return E_OUTOFMEMORY;
-    }
 
     hr = vertexdeclaration_init(object, device, elements, element_count, parent, parent_ops);
     if (FAILED(hr))
@@ -315,7 +312,7 @@ static unsigned int convert_fvf_to_declaration(const struct wined3d_gl_info *gl_
     BOOL has_specular = !!(fvf & WINED3DFVF_SPECULAR);
 
     DWORD num_textures = (fvf & WINED3DFVF_TEXCOUNT_MASK) >> WINED3DFVF_TEXCOUNT_SHIFT;
-    DWORD texcoords = (fvf & 0xFFFF0000) >> 16;
+    DWORD texcoords = (fvf & 0xffff0000) >> 16;
     struct wined3d_fvf_convert_state state;
     unsigned int size;
     unsigned int idx;
