@@ -48,10 +48,6 @@ struct wined3d_wndproc_table
 
 static struct wined3d_wndproc_table wndproc_table;
 
-int num_lock = 0;
-void (CDECL *wine_tsx11_lock_ptr)(void) = NULL;
-void (CDECL *wine_tsx11_unlock_ptr)(void) = NULL;
-
 static CRITICAL_SECTION wined3d_cs;
 static CRITICAL_SECTION_DEBUG wined3d_cs_debug =
 {
@@ -76,8 +72,6 @@ static CRITICAL_SECTION wined3d_wndproc_cs = {&wined3d_wndproc_cs_debug, -1, 0,
  * where appropriate. */
 struct wined3d_settings wined3d_settings =
 {
-    VS_HW,          /* Hardware by default */
-    PS_HW,          /* Hardware by default */
     TRUE,           /* Use of GLSL enabled by default */
     ORM_FBO,        /* Use FBOs to do offscreen rendering */
     RTL_READTEX,    /* Default render target locking method */
@@ -87,7 +81,10 @@ struct wined3d_settings wined3d_settings =
     NULL,           /* No wine logo by default */
     TRUE,           /* Multisampling enabled by default. */
     FALSE,          /* No strict draw ordering. */
-    FALSE,          /* Try to render onscreen by default. */
+    TRUE,           /* Don't try to render onscreen by default. */
+    ~0U,            /* No VS shader model limit by default. */
+    ~0U,            /* No GS shader model limit by default. */
+    ~0U,            /* No PS shader model limit by default. */
 };
 
 /* Do not call while under the GL lock. */
@@ -132,14 +129,9 @@ static DWORD get_config_key_dword(HKEY defkey, HKEY appkey, const char *name, DW
     return ERROR_FILE_NOT_FOUND;
 }
 
-static void CDECL wined3d_do_nothing(void)
-{
-}
-
 static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
 {
     DWORD wined3d_context_tls_idx;
-    HMODULE mod;
     char buffer[MAX_PATH+10];
     DWORD size = sizeof(buffer);
     HKEY hkey = 0;
@@ -183,17 +175,6 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
 
     DisableThreadLibraryCalls(hInstDLL);
 
-    mod = GetModuleHandleA( "winex11.drv" );
-    if (mod)
-    {
-        wine_tsx11_lock_ptr   = (void *)GetProcAddress( mod, "wine_tsx11_lock" );
-        wine_tsx11_unlock_ptr = (void *)GetProcAddress( mod, "wine_tsx11_unlock" );
-    }
-    else /* We are most likely on Windows */
-    {
-        wine_tsx11_lock_ptr   = wined3d_do_nothing;
-        wine_tsx11_unlock_ptr = wined3d_do_nothing;
-    }
     /* @@ Wine registry key: HKCU\Software\Wine\Direct3D */
     if ( RegOpenKeyA( HKEY_CURRENT_USER, "Software\\Wine\\Direct3D", &hkey ) ) hkey = 0;
 
@@ -216,27 +197,6 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
 
     if (hkey || appkey)
     {
-        if ( !get_config_key( hkey, appkey, "VertexShaderMode", buffer, size) )
-        {
-            if (!strcmp(buffer,"none"))
-            {
-                TRACE("Disable vertex shaders\n");
-                wined3d_settings.vs_mode = VS_NONE;
-            }
-        }
-        if ( !get_config_key( hkey, appkey, "PixelShaderMode", buffer, size) )
-        {
-            if (!strcmp(buffer,"enabled"))
-            {
-                TRACE("Allow pixel shaders\n");
-                wined3d_settings.ps_mode = PS_HW;
-            }
-            if (!strcmp(buffer,"disabled"))
-            {
-                TRACE("Disable pixel shaders\n");
-                wined3d_settings.ps_mode = PS_NONE;
-            }
-        }
         if ( !get_config_key( hkey, appkey, "UseGLSL", buffer, size) )
         {
             if (!strcmp(buffer,"disabled"))
@@ -338,18 +298,18 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
             wined3d_settings.strict_draw_ordering = TRUE;
         }
         if (!get_config_key(hkey, appkey, "AlwaysOffscreen", buffer, size)
-                && !strcmp(buffer,"enabled"))
+                && !strcmp(buffer,"disabled"))
         {
-            TRACE("Always rendering backbuffers offscreen.\n");
-            wined3d_settings.always_offscreen = TRUE;
+            TRACE("Not always rendering backbuffers offscreen.\n");
+            wined3d_settings.always_offscreen = FALSE;
         }
+        if (!get_config_key_dword(hkey, appkey, "MaxShaderModelVS", &wined3d_settings.max_sm_vs))
+            TRACE("Limiting VS shader model to %u.\n", wined3d_settings.max_sm_vs);
+        if (!get_config_key_dword(hkey, appkey, "MaxShaderModelGS", &wined3d_settings.max_sm_gs))
+            TRACE("Limiting GS shader model to %u.\n", wined3d_settings.max_sm_gs);
+        if (!get_config_key_dword(hkey, appkey, "MaxShaderModelPS", &wined3d_settings.max_sm_ps))
+            TRACE("Limiting PS shader model to %u.\n", wined3d_settings.max_sm_ps);
     }
-    if (wined3d_settings.vs_mode == VS_HW)
-        TRACE("Allow HW vertex shaders\n");
-    if (wined3d_settings.ps_mode == PS_NONE)
-        TRACE("Disable pixel shaders\n");
-    if (wined3d_settings.glslRequested)
-        TRACE("If supported by your system, GL Shading Language will be used\n");
 
     if (appkey) RegCloseKey( appkey );
     if (hkey) RegCloseKey( hkey );
