@@ -26,12 +26,21 @@
 #define ERR(...) do { if (macdrv_err_on) LogError(__func__, __VA_ARGS__); } while (false)
 
 
+enum {
+    WineApplicationEventWakeQuery,
+};
+
+
 @class WineEventQueue;
 @class WineWindow;
 
 
-@interface WineApplication : NSApplication <NSApplicationDelegate>
+@interface WineApplicationController : NSObject <NSApplicationDelegate>
 {
+    CFRunLoopSourceRef requestSource;
+    NSMutableArray* requests;
+    dispatch_queue_t requestsManipQueue;
+
     NSMutableArray* eventQueues;
     NSLock*         eventQueuesLock;
 
@@ -43,13 +52,41 @@
 
     CGEventSourceKeyboardType keyboardType;
     NSEvent* lastFlagsChanged;
+    BOOL inputSourceIsInputMethod;
+    BOOL inputSourceIsInputMethodValid;
 
     CGFloat primaryScreenHeight;
     BOOL primaryScreenHeightValid;
+
+    WineWindow* lastTargetWindow;
+    BOOL forceNextMouseMoveAbsolute;
+
+    NSMutableArray* orderedWineWindows;
+
+    NSMutableDictionary* originalDisplayModes;
+
+    NSArray*    cursorFrames;
+    int         cursorFrame;
+    NSTimer*    cursorTimer;
+    BOOL        cursorHidden;
+
+    BOOL clippingCursor;
+    CGRect cursorClipRect;
+    CFMachPortRef cursorClippingEventTap;
+    NSMutableArray* warpRecords;
+    CGPoint synthesizedLocation;
+    NSTimeInterval lastSetCursorPositionTime;
+    NSTimeInterval lastEventTapEventTime;
+
+    NSImage* applicationIcon;
 }
 
 @property (nonatomic) CGEventSourceKeyboardType keyboardType;
 @property (readonly, copy, nonatomic) NSEvent* lastFlagsChanged;
+@property (readonly, nonatomic) NSArray* orderedWineWindows;
+@property (readonly, nonatomic) BOOL areDisplaysCaptured;
+
+    + (WineApplicationController*) sharedController;
 
     - (void) transformProcessToForeground;
 
@@ -61,11 +98,32 @@
 
     - (void) windowGotFocus:(WineWindow*)window;
 
+    - (BOOL) waitUntilQueryDone:(int*)done timeout:(NSDate*)timeout processEvents:(BOOL)processEvents;
+
     - (void) keyboardSelectionDidChange;
 
+    - (void) flipRect:(NSRect*)rect;
+
+    - (void) wineWindow:(WineWindow*)window
+                ordered:(NSWindowOrderingMode)order
+             relativeTo:(WineWindow*)otherWindow;
+
+    - (BOOL) handleEvent:(NSEvent*)anEvent;
+    - (void) didSendEvent:(NSEvent*)anEvent;
+
 @end
 
-void OnMainThread(dispatch_block_t block);
+
+@interface WineApplication : NSApplication
+{
+    WineApplicationController* wineController;
+}
+
+@property (readwrite, assign, nonatomic) WineApplicationController* wineController;
+
+@end
+
+
 void OnMainThreadAsync(dispatch_block_t block);
 
 void LogError(const char* func, NSString* format, ...);
