@@ -48,14 +48,26 @@
 
     NSUInteger lastModifierFlags;
 
-    BOOL forceNextMouseMoveAbsolute;
     double mouseMoveDeltaX, mouseMoveDeltaY;
 
+    NSInteger levelWhenActive;
+
+    NSTimer* liveResizeDisplayTimer;
+
+    void* imeData;
+    BOOL commandDone;
+
     BOOL causing_becomeKeyWindow;
     BOOL ignore_windowMiniaturize;
     BOOL ignore_windowDeminiaturize;
 }
 
 @property (retain, readonly, nonatomic) WineEventQueue* queue;
+@property (readonly, nonatomic) BOOL floating;
+@property (readonly, nonatomic) NSInteger levelWhenActive;
+
+    - (void) adjustWindowLevel;
+
+    - (void) postMouseMovedEvent:(NSEvent *)theEvent absolute:(BOOL)absolute;
 
 @end
