@@ -2,7 +2,7 @@
  * Mac graphics driver initialisation functions
  *
  * Copyright 1996 Alexandre Julliard
- * Copyright 2011, 2012 Ken Thomases for CodeWeavers, Inc.
+ * Copyright 2011, 2012, 2013 Ken Thomases for CodeWeavers, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -200,6 +200,14 @@ static void device_init(void)
 }
 
 
+void macdrv_reset_device_metrics(void)
+{
+    EnterCriticalSection(&device_data_section);
+    device_data_valid = FALSE;
+    LeaveCriticalSection(&device_data_section);
+}
+
+
 static MACDRV_PDEVICE *create_mac_physdev(void)
 {
     MACDRV_PDEVICE *physDev;
@@ -453,7 +461,7 @@ static const struct gdi_dc_funcs macdrv_funcs =
     NULL,                                   /* pGetCharABCWidthsI */
     NULL,                                   /* pGetCharWidth */
     macdrv_GetDeviceCaps,                   /* pGetDeviceCaps */
-    NULL,                                   /* pGetDeviceGammaRamp */
+    macdrv_GetDeviceGammaRamp,              /* pGetDeviceGammaRamp */
     NULL,                                   /* pGetFontData */
     NULL,                                   /* pGetFontUnicodeRanges */
     NULL,                                   /* pGetGlyphIndices */
@@ -514,7 +522,7 @@ static const struct gdi_dc_funcs macdrv_funcs =
     NULL,                                   /* pSetDCPenColor */
     NULL,                                   /* pSetDIBitsToDevice */
     NULL,                                   /* pSetDeviceClipping */
-    NULL,                                   /* pSetDeviceGammaRamp */
+    macdrv_SetDeviceGammaRamp,              /* pSetDeviceGammaRamp */
     NULL,                                   /* pSetLayout */
     NULL,                                   /* pSetMapMode */
     NULL,                                   /* pSetMapperFlags */
@@ -540,7 +548,7 @@ static const struct gdi_dc_funcs macdrv_funcs =
     NULL,                                   /* pStrokePath */
     NULL,                                   /* pUnrealizePalette */
     NULL,                                   /* pWidenPath */
-    NULL,                                   /* wine_get_wgl_driver */
+    macdrv_wine_get_wgl_driver,             /* wine_get_wgl_driver */
     GDI_PRIORITY_GRAPHICS_DRV               /* priority */
 };
 
