@@ -35,6 +35,8 @@
 #include "wine/gdi_driver.h"
 
 
+extern const char* debugstr_cf(CFTypeRef t) DECLSPEC_HIDDEN;
+
 static inline CGRect cgrect_from_rect(RECT rect)
 {
     if (rect.left >= rect.right || rect.top >= rect.bottom)
@@ -62,12 +64,17 @@ static inline const char *wine_dbgstr_cgrect(CGRect cgrect)
                             CGRectGetMaxX(cgrect), CGRectGetMaxY(cgrect));
 }
 
+extern const char* debugstr_cf(CFTypeRef t) DECLSPEC_HIDDEN;
+
 
 /**************************************************************************
  * Mac GDI driver
  */
 
 extern CGRect macdrv_get_desktop_rect(void) DECLSPEC_HIDDEN;
+extern void macdrv_reset_device_metrics(void) DECLSPEC_HIDDEN;
+extern BOOL macdrv_GetDeviceGammaRamp(PHYSDEV dev, LPVOID ramp) DECLSPEC_HIDDEN;
+extern BOOL macdrv_SetDeviceGammaRamp(PHYSDEV dev, LPVOID ramp) DECLSPEC_HIDDEN;
 
 
 /**************************************************************************
@@ -78,6 +85,10 @@ extern CGRect macdrv_get_desktop_rect(void) DECLSPEC_HIDDEN;
 enum macdrv_window_messages
 {
     WM_MACDRV_SET_WIN_REGION = 0x80001000,
+    WM_MACDRV_UPDATE_DESKTOP_RECT,
+    WM_MACDRV_RESET_DEVICE_METRICS,
+    WM_MACDRV_DISPLAYCHANGE,
+    WM_MACDRV_ACTIVATE_ON_FOLLOWING_FOCUS,
 };
 
 struct macdrv_thread_data
@@ -112,25 +123,37 @@ struct macdrv_win_data
     RECT                window_rect;            /* USER window rectangle relative to parent */
     RECT                whole_rect;             /* Mac window rectangle for the whole window relative to parent */
     RECT                client_rect;            /* client area relative to parent */
+    int                 pixel_format;           /* pixel format for GL */
+    macdrv_view         gl_view;                /* view for GL */
+    RECT                gl_rect;                /* GL view rectangle relative to whole_rect */
     COLORREF            color_key;              /* color key for layered window; CLR_INVALID is not color keyed */
     BOOL                on_screen : 1;          /* is window ordered in? (minimized or not) */
     BOOL                shaped : 1;             /* is window using a custom region shape? */
     BOOL                layered : 1;            /* is window layered and with valid attributes? */
+    BOOL                ulw_layered : 1;        /* has UpdateLayeredWindow() been called for window? */
     BOOL                per_pixel_alpha : 1;    /* is window using per-pixel alpha? */
     BOOL                minimized : 1;          /* is window minimized? */
     struct window_surface *surface;
 };
 
+extern struct macdrv_win_data *get_win_data(HWND hwnd) DECLSPEC_HIDDEN;
+extern void release_win_data(struct macdrv_win_data *data) DECLSPEC_HIDDEN;
+extern macdrv_window macdrv_get_cocoa_window(HWND hwnd, BOOL require_on_screen) DECLSPEC_HIDDEN;
 extern RGNDATA *get_region_data(HRGN hrgn, HDC hdc_lptodp) DECLSPEC_HIDDEN;
-extern struct window_surface *create_surface(macdrv_window window, const RECT *rect, BOOL use_alpha) DECLSPEC_HIDDEN;
+extern struct window_surface *create_surface(macdrv_window window, const RECT *rect,
+                                             struct window_surface *old_surface, BOOL use_alpha) DECLSPEC_HIDDEN;
 extern void set_window_surface(macdrv_window window, struct window_surface *window_surface) DECLSPEC_HIDDEN;
 extern void set_surface_use_alpha(struct window_surface *window_surface, BOOL use_alpha) DECLSPEC_HIDDEN;
+extern void surface_clip_to_visible_rect(struct window_surface *window_surface, const RECT *visible_rect) DECLSPEC_HIDDEN;
+
+extern void macdrv_handle_event(const macdrv_event *event) DECLSPEC_HIDDEN;
 
 extern void macdrv_window_close_requested(HWND hwnd) DECLSPEC_HIDDEN;
 extern void macdrv_window_frame_changed(HWND hwnd, CGRect frame) DECLSPEC_HIDDEN;
 extern void macdrv_window_got_focus(HWND hwnd, const macdrv_event *event) DECLSPEC_HIDDEN;
 extern void macdrv_window_lost_focus(HWND hwnd, const macdrv_event *event) DECLSPEC_HIDDEN;
 extern void macdrv_app_deactivated(void) DECLSPEC_HIDDEN;
+extern void macdrv_app_quit_requested(const macdrv_event *event) DECLSPEC_HIDDEN;
 extern void macdrv_window_did_minimize(HWND hwnd) DECLSPEC_HIDDEN;
 extern void macdrv_window_did_unminimize(HWND hwnd) DECLSPEC_HIDDEN;
 
@@ -142,4 +165,43 @@ extern void macdrv_compute_keyboard_layout(struct macdrv_thread_data *thread_dat
 extern void macdrv_keyboard_changed(const macdrv_event *event) DECLSPEC_HIDDEN;
 extern void macdrv_key_event(HWND hwnd, const macdrv_event *event) DECLSPEC_HIDDEN;
 
+extern void macdrv_displays_changed(const macdrv_event *event) DECLSPEC_HIDDEN;
+
+extern void macdrv_clipboard_process_attach(void) DECLSPEC_HIDDEN;
+extern BOOL query_pasteboard_data(HWND hwnd, CFStringRef type) DECLSPEC_HIDDEN;
+extern const char *debugstr_format(UINT id) DECLSPEC_HIDDEN;
+extern HANDLE macdrv_get_pasteboard_data(CFTypeRef pasteboard, UINT desired_format) DECLSPEC_HIDDEN;
+extern BOOL CDECL macdrv_pasteboard_has_format(CFTypeRef pasteboard, UINT desired_format) DECLSPEC_HIDDEN;
+extern CFArrayRef macdrv_copy_pasteboard_formats(CFTypeRef pasteboard) DECLSPEC_HIDDEN;
+
+extern BOOL query_drag_operation(macdrv_query* query) DECLSPEC_HIDDEN;
+extern BOOL query_drag_exited(macdrv_query* query) DECLSPEC_HIDDEN;
+extern BOOL query_drag_drop(macdrv_query* query) DECLSPEC_HIDDEN;
+
+extern struct opengl_funcs *macdrv_wine_get_wgl_driver(PHYSDEV dev, UINT version) DECLSPEC_HIDDEN;
+extern void sync_gl_view(struct macdrv_win_data *data) DECLSPEC_HIDDEN;
+extern void set_gl_view_parent(HWND hwnd, HWND parent) DECLSPEC_HIDDEN;
+
+extern CGImageRef create_cgimage_from_icon_bitmaps(HDC hdc, HANDLE icon, HBITMAP hbmColor,
+                                                   unsigned char *color_bits, int color_size, HBITMAP hbmMask,
+                                                   unsigned char *mask_bits, int mask_size, int width,
+                                                   int height, int istep) DECLSPEC_HIDDEN;
+extern CGImageRef create_cgimage_from_icon(HANDLE icon, int width, int height) DECLSPEC_HIDDEN;
+extern CFArrayRef create_app_icon_images(void) DECLSPEC_HIDDEN;
+
+extern void macdrv_status_item_clicked(const macdrv_event *event) DECLSPEC_HIDDEN;
+
+
+/**************************************************************************
+ * Mac IME driver
+ */
+
+extern void IME_RegisterClasses(HINSTANCE hImeInst) DECLSPEC_HIDDEN;
+
+extern BOOL macdrv_process_text_input(UINT vkey, UINT scan, UINT repeat, const BYTE *key_state,
+                                      void *himc) DECLSPEC_HIDDEN;
+
+extern void macdrv_im_set_cursor_pos(const macdrv_event *event) DECLSPEC_HIDDEN;
+extern void macdrv_im_set_text(const macdrv_event *event) DECLSPEC_HIDDEN;
+
 #endif  /* __WINE_MACDRV_H */
