@@ -99,9 +99,26 @@
 #endif
 
 
+/* Must match the values of Cocoa's NSDragOperation enum. */
+enum {
+    DRAG_OP_NONE    = 0,
+    DRAG_OP_COPY    = 1,
+    DRAG_OP_LINK    = 2,
+    DRAG_OP_GENERIC = 4,
+    DRAG_OP_PRIVATE = 8,
+    DRAG_OP_MOVE    = 16,
+    DRAG_OP_DELETE  = 32,
+    DRAG_OP_EVERY   = UINT32_MAX
+};
+
+
 typedef struct macdrv_opaque_window* macdrv_window;
 typedef struct macdrv_opaque_event_queue* macdrv_event_queue;
+typedef struct macdrv_opaque_view* macdrv_view;
+typedef struct macdrv_opaque_opengl_context* macdrv_opengl_context;
+typedef struct macdrv_opaque_status_item* macdrv_status_item;
 struct macdrv_event;
+struct macdrv_query;
 
 struct macdrv_display {
     CGDirectDisplayID displayID;
@@ -116,16 +133,32 @@ extern int macdrv_err_on;
 extern int macdrv_start_cocoa_app(unsigned long long tickcount) DECLSPEC_HIDDEN;
 extern void macdrv_window_rejected_focus(const struct macdrv_event *event) DECLSPEC_HIDDEN;
 extern void macdrv_beep(void) DECLSPEC_HIDDEN;
+extern void macdrv_set_application_icon(CFArrayRef images) DECLSPEC_HIDDEN;
+extern void macdrv_quit_reply(int reply) DECLSPEC_HIDDEN;
+extern int macdrv_using_input_method(void) DECLSPEC_HIDDEN;
+
+
+/* cursor */
+extern void macdrv_set_cursor(CFStringRef name, CFArrayRef frames) DECLSPEC_HIDDEN;
+extern int macdrv_get_cursor_position(CGPoint *pos) DECLSPEC_HIDDEN;
+extern int macdrv_set_cursor_position(CGPoint pos) DECLSPEC_HIDDEN;
+extern int macdrv_clip_cursor(CGRect rect) DECLSPEC_HIDDEN;
 
 
 /* display */
 extern int macdrv_get_displays(struct macdrv_display** displays, int* count) DECLSPEC_HIDDEN;
 extern void macdrv_free_displays(struct macdrv_display* displays) DECLSPEC_HIDDEN;
+extern int macdrv_set_display_mode(const struct macdrv_display* display,
+                                   CGDisplayModeRef display_mode) DECLSPEC_HIDDEN;
 
 
 /* event */
 enum {
     APP_DEACTIVATED,
+    APP_QUIT_REQUESTED,
+    DISPLAYS_CHANGED,
+    IM_SET_CURSOR_POS,
+    IM_SET_TEXT,
     KEY_PRESS,
     KEY_RELEASE,
     KEYBOARD_CHANGED,
@@ -133,6 +166,8 @@ enum {
     MOUSE_MOVED,
     MOUSE_MOVED_ABSOLUTE,
     MOUSE_SCROLL,
+    QUERY_EVENT,
+    STATUS_ITEM_CLICKED,
     WINDOW_CLOSE_REQUESTED,
     WINDOW_DID_MINIMIZE,
     WINDOW_DID_UNMINIMIZE,
@@ -142,12 +177,36 @@ enum {
     NUM_EVENT_TYPES
 };
 
+enum {
+    QUIT_REASON_NONE,
+    QUIT_REASON_LOGOUT,
+    QUIT_REASON_RESTART,
+    QUIT_REASON_SHUTDOWN,
+};
+
 typedef uint32_t macdrv_event_mask;
 
 typedef struct macdrv_event {
+    int                 refs;
+    int                 deliver;
     int                 type;
     macdrv_window       window;
     union {
+        struct {
+            int reason;
+        }                                           app_quit_requested;
+        struct {
+            int activating;
+        }                                           displays_changed;
+        struct {
+            void           *data;
+            unsigned int    pos;
+        }                                           im_set_cursor_pos;
+        struct {
+            void           *data;
+            CFStringRef     text;       /* new text or NULL if just completing existing text */
+            unsigned int    complete;   /* is completing text? */
+        }                                           im_set_text;
         struct {
             CGKeyCode                   keycode;
             CGEventFlags                modifiers;
@@ -177,6 +236,13 @@ typedef struct macdrv_event {
             int             y;
             unsigned long   time_ms;
         }                                           mouse_scroll;
+        struct {
+            struct macdrv_query *query;
+        }                                           query_event;
+        struct {
+            macdrv_status_item  item;
+            int                 count;
+        }                                           status_item_clicked;
         struct {
             CGRect frame;
         }                                           window_frame_changed;
@@ -187,18 +253,59 @@ typedef struct macdrv_event {
     };
 } macdrv_event;
 
+enum {
+    QUERY_DRAG_DROP,
+    QUERY_DRAG_EXITED,
+    QUERY_DRAG_OPERATION,
+    QUERY_PASTEBOARD_DATA,
+    NUM_QUERY_TYPES
+};
+
+typedef struct macdrv_query {
+    int                 refs;
+    int                 type;
+    macdrv_window       window;
+    int                 status;
+    int                 done;
+    union {
+        struct {
+            int                 x;
+            int                 y;
+            uint32_t            op;
+            CFTypeRef           pasteboard;
+        }                                           drag_drop;
+        struct {
+            int                 x;
+            int                 y;
+            uint32_t            offered_ops;
+            uint32_t            accepted_op;
+            CFTypeRef           pasteboard;
+        }                                           drag_operation;
+        struct {
+            CFStringRef type;
+        }                                           pasteboard_data;
+    };
+} macdrv_query;
+
 static inline macdrv_event_mask event_mask_for_type(int type)
 {
     return ((macdrv_event_mask)1 << type);
 }
 
-extern macdrv_event_queue macdrv_create_event_queue(void) DECLSPEC_HIDDEN;
+typedef void (*macdrv_event_handler)(const macdrv_event *event);
+
+extern macdrv_event_queue macdrv_create_event_queue(macdrv_event_handler handler) DECLSPEC_HIDDEN;
 extern void macdrv_destroy_event_queue(macdrv_event_queue queue) DECLSPEC_HIDDEN;
 extern int macdrv_get_event_queue_fd(macdrv_event_queue queue) DECLSPEC_HIDDEN;
 
-extern int macdrv_get_event_from_queue(macdrv_event_queue queue,
-        macdrv_event_mask mask, macdrv_event *event) DECLSPEC_HIDDEN;
-extern void macdrv_cleanup_event(macdrv_event *event) DECLSPEC_HIDDEN;
+extern int macdrv_copy_event_from_queue(macdrv_event_queue queue,
+        macdrv_event_mask mask, macdrv_event **event) DECLSPEC_HIDDEN;
+extern void macdrv_release_event(macdrv_event *event) DECLSPEC_HIDDEN;
+
+extern macdrv_query* macdrv_create_query(void) DECLSPEC_HIDDEN;
+extern macdrv_query* macdrv_retain_query(macdrv_query *query) DECLSPEC_HIDDEN;
+extern void macdrv_release_query(macdrv_query *query) DECLSPEC_HIDDEN;
+extern void macdrv_set_query_done(macdrv_query *query) DECLSPEC_HIDDEN;
 
 
 /* window */
@@ -238,7 +345,7 @@ extern void macdrv_get_cocoa_window_frame(macdrv_window w, CGRect* out_frame) DE
 extern void macdrv_set_cocoa_parent_window(macdrv_window w, macdrv_window parent) DECLSPEC_HIDDEN;
 extern void macdrv_set_window_surface(macdrv_window w, void *surface, pthread_mutex_t *mutex) DECLSPEC_HIDDEN;
 extern CGImageRef create_surface_image(void *window_surface, CGRect *rect, int copy_data) DECLSPEC_HIDDEN;
-extern int get_surface_region_rects(void *window_surface, const CGRect **rects, int *count) DECLSPEC_HIDDEN;
+extern int get_surface_blit_rects(void *window_surface, const CGRect **rects, int *count) DECLSPEC_HIDDEN;
 extern void macdrv_window_needs_display(macdrv_window w, CGRect rect) DECLSPEC_HIDDEN;
 extern void macdrv_set_window_shape(macdrv_window w, const CGRect *rects, int count) DECLSPEC_HIDDEN;
 extern void macdrv_set_window_alpha(macdrv_window w, CGFloat alpha) DECLSPEC_HIDDEN;
@@ -246,10 +353,41 @@ extern void macdrv_set_window_color_key(macdrv_window w, CGFloat keyRed, CGFloat
                                         CGFloat keyBlue) DECLSPEC_HIDDEN;
 extern void macdrv_clear_window_color_key(macdrv_window w) DECLSPEC_HIDDEN;
 extern void macdrv_window_use_per_pixel_alpha(macdrv_window w, int use_per_pixel_alpha) DECLSPEC_HIDDEN;
-extern void macdrv_give_cocoa_window_focus(macdrv_window w) DECLSPEC_HIDDEN;
+extern void macdrv_give_cocoa_window_focus(macdrv_window w, int activate) DECLSPEC_HIDDEN;
+extern macdrv_view macdrv_create_view(macdrv_window w, CGRect rect) DECLSPEC_HIDDEN;
+extern void macdrv_dispose_view(macdrv_view v) DECLSPEC_HIDDEN;
+extern void macdrv_set_view_window_and_frame(macdrv_view v, macdrv_window w, CGRect rect) DECLSPEC_HIDDEN;
+extern void macdrv_add_view_opengl_context(macdrv_view v, macdrv_opengl_context c) DECLSPEC_HIDDEN;
+extern void macdrv_remove_view_opengl_context(macdrv_view v, macdrv_opengl_context c) DECLSPEC_HIDDEN;
+extern uint32_t macdrv_window_background_color(void) DECLSPEC_HIDDEN;
+extern int macdrv_send_text_input_event(int pressed, unsigned int flags, int repeat, int keyc,
+                                        void* data) DECLSPEC_HIDDEN;
 
 
 /* keyboard */
 extern CFDataRef macdrv_copy_keyboard_layout(CGEventSourceKeyboardType* keyboard_type, int* is_iso) DECLSPEC_HIDDEN;
 
+
+/* clipboard */
+extern CFArrayRef macdrv_copy_pasteboard_types(CFTypeRef pasteboard) DECLSPEC_HIDDEN;
+extern CFDataRef macdrv_copy_pasteboard_data(CFTypeRef pasteboard, CFStringRef type) DECLSPEC_HIDDEN;
+extern int macdrv_is_pasteboard_owner(void) DECLSPEC_HIDDEN;
+extern void macdrv_clear_pasteboard(void) DECLSPEC_HIDDEN;
+extern int macdrv_set_pasteboard_data(CFStringRef type, CFDataRef data, macdrv_window w) DECLSPEC_HIDDEN;
+
+
+/* opengl */
+extern macdrv_opengl_context macdrv_create_opengl_context(void* cglctx) DECLSPEC_HIDDEN;
+extern void macdrv_dispose_opengl_context(macdrv_opengl_context c) DECLSPEC_HIDDEN;
+extern void macdrv_make_context_current(macdrv_opengl_context c, macdrv_view v) DECLSPEC_HIDDEN;
+extern void macdrv_update_opengl_context(macdrv_opengl_context c) DECLSPEC_HIDDEN;
+extern void macdrv_flush_opengl_context(macdrv_opengl_context c) DECLSPEC_HIDDEN;
+
+
+/* systray / status item */
+extern macdrv_status_item macdrv_create_status_item(macdrv_event_queue q) DECLSPEC_HIDDEN;
+extern void macdrv_destroy_status_item(macdrv_status_item s) DECLSPEC_HIDDEN;
+extern void macdrv_set_status_item_image(macdrv_status_item s, CGImageRef cgimage) DECLSPEC_HIDDEN;
+extern void macdrv_set_status_item_tooltip(macdrv_status_item s, CFStringRef cftip) DECLSPEC_HIDDEN;
+
 #endif  /* __WINE_MACDRV_COCOA_H */
