@@ -42,6 +42,8 @@ static CRITICAL_SECTION win_data_section = { &critsect_debug, -1, 0, 0, 0, 0 };
 
 static CFMutableDictionaryRef win_datas;
 
+DWORD activate_on_focus_time;
+
 
 void CDECL macdrv_SetFocus(HWND hwnd);
 
@@ -150,35 +152,6 @@ static void get_mac_rect_offset(struct macdrv_win_data *data, DWORD style, RECT
 }
 
 
-/***********************************************************************
- *              show_window
- */
-static void show_window(struct macdrv_win_data *data)
-{
-    TRACE("win %p/%p\n", data->hwnd, data->cocoa_window);
-
-    data->on_screen = macdrv_order_cocoa_window(data->cocoa_window, NULL, NULL);
-    if (data->on_screen)
-    {
-        HWND hwndFocus = GetFocus();
-        if (hwndFocus && (data->hwnd == hwndFocus || IsChild(data->hwnd, hwndFocus)))
-            macdrv_SetFocus(hwndFocus);
-    }
-}
-
-
-/***********************************************************************
- *              hide_window
- */
-static void hide_window(struct macdrv_win_data *data)
-{
-    TRACE("win %p/%p\n", data->hwnd, data->cocoa_window);
-
-    macdrv_hide_cocoa_window(data->cocoa_window);
-    data->on_screen = FALSE;
-}
-
-
 /***********************************************************************
  *              macdrv_window_to_mac_rect
  *
@@ -274,7 +247,7 @@ static struct macdrv_win_data *alloc_win_data(HWND hwnd)
  *
  * Lock and return the data structure associated with a window.
  */
-static struct macdrv_win_data *get_win_data(HWND hwnd)
+struct macdrv_win_data *get_win_data(HWND hwnd)
 {
     struct macdrv_win_data *data;
 
@@ -292,7 +265,7 @@ static struct macdrv_win_data *get_win_data(HWND hwnd)
  *
  * Release the data returned by get_win_data.
  */
-static void release_win_data(struct macdrv_win_data *data)
+void release_win_data(struct macdrv_win_data *data)
 {
     if (data) LeaveCriticalSection(&win_data_section);
 }
@@ -303,10 +276,12 @@ static void release_win_data(struct macdrv_win_data *data)
  *
  * Return the Mac window associated with the full area of a window
  */
-static macdrv_window macdrv_get_cocoa_window(HWND hwnd)
+macdrv_window macdrv_get_cocoa_window(HWND hwnd, BOOL require_on_screen)
 {
     struct macdrv_win_data *data = get_win_data(hwnd);
-    macdrv_window ret = data ? data->cocoa_window : NULL;
+    macdrv_window ret = NULL;
+    if (data && (data->on_screen || !require_on_screen))
+        ret = data->cocoa_window;
     release_win_data(data);
     return ret;
 }
@@ -330,7 +305,7 @@ static void set_cocoa_window_properties(struct macdrv_win_data *data)
     ex_style = GetWindowLongW(data->hwnd, GWL_EXSTYLE);
 
     owner = GetWindow(data->hwnd, GW_OWNER);
-    owner_win = macdrv_get_cocoa_window(owner);
+    owner_win = macdrv_get_cocoa_window(owner, TRUE);
     macdrv_set_cocoa_parent_window(data->cocoa_window, owner_win);
 
     get_cocoa_window_features(data, style, ex_style, &wf);
@@ -602,6 +577,55 @@ static struct macdrv_win_data *macdrv_create_win_data(HWND hwnd, const RECT *win
 }
 
 
+/***********************************************************************
+ *              show_window
+ */
+static void show_window(struct macdrv_win_data *data)
+{
+    HWND prev = NULL;
+    HWND next = NULL;
+    macdrv_window prev_window = NULL;
+    macdrv_window next_window = NULL;
+
+    /* find window that this one must be after */
+    prev = GetWindow(data->hwnd, GW_HWNDPREV);
+    while (prev && !((GetWindowLongW(prev, GWL_STYLE) & WS_VISIBLE) &&
+                     (prev_window = macdrv_get_cocoa_window(prev, TRUE))))
+        prev = GetWindow(prev, GW_HWNDPREV);
+    if (!prev_window)
+    {
+        /* find window that this one must be before */
+        next = GetWindow(data->hwnd, GW_HWNDNEXT);
+        while (next && !((GetWindowLongW(next, GWL_STYLE) & WS_VISIBLE) &&
+                         (next_window = macdrv_get_cocoa_window(next, TRUE))))
+            next = GetWindow(next, GW_HWNDNEXT);
+    }
+
+    TRACE("win %p/%p below %p/%p above %p/%p\n",
+          data->hwnd, data->cocoa_window, prev, prev_window, next, next_window);
+
+    data->on_screen = macdrv_order_cocoa_window(data->cocoa_window, prev_window, next_window);
+    if (data->on_screen)
+    {
+        HWND hwndFocus = GetFocus();
+        if (hwndFocus && (data->hwnd == hwndFocus || IsChild(data->hwnd, hwndFocus)))
+            macdrv_SetFocus(hwndFocus);
+    }
+}
+
+
+/***********************************************************************
+ *              hide_window
+ */
+static void hide_window(struct macdrv_win_data *data)
+{
+    TRACE("win %p/%p\n", data->hwnd, data->cocoa_window);
+
+    macdrv_hide_cocoa_window(data->cocoa_window);
+    data->on_screen = FALSE;
+}
+
+
 /***********************************************************************
  *              get_region_data
  *
@@ -690,30 +714,7 @@ static void sync_window_position(struct macdrv_win_data *data, UINT swp_flags)
           wine_dbgstr_rect(&data->whole_rect));
 
     if (data->on_screen && (!(swp_flags & SWP_NOZORDER) || (swp_flags & SWP_SHOWWINDOW)))
-    {
-        HWND next = NULL;
-        macdrv_window prev_window = NULL;
-        macdrv_window next_window = NULL;
-
-        /* find window that this one must be after */
-        HWND prev = GetWindow(data->hwnd, GW_HWNDPREV);
-        while (prev && !((GetWindowLongW(prev, GWL_STYLE) & WS_VISIBLE) &&
-                         (prev_window = macdrv_get_cocoa_window(prev))))
-            prev = GetWindow(prev, GW_HWNDPREV);
-        if (!prev_window)
-        {
-            /* find window that this one must be before */
-            next = GetWindow(data->hwnd, GW_HWNDNEXT);
-            while (next && !((GetWindowLongW(next, GWL_STYLE) & WS_VISIBLE) &&
-                             (next_window = macdrv_get_cocoa_window(next))))
-                next = GetWindow(next, GW_HWNDNEXT);
-        }
-
-        data->on_screen = macdrv_order_cocoa_window(data->cocoa_window, prev_window, next_window);
-
-        TRACE("win %p/%p below %p/%p above %p/%p\n",
-              data->hwnd, data->cocoa_window, prev, prev_window, next, next_window);
-    }
+        show_window(data);
 }
 
 
@@ -827,6 +828,7 @@ void CDECL macdrv_DestroyWindow(HWND hwnd)
 
     if (!(data = get_win_data(hwnd))) return;
 
+    if (data->gl_view) macdrv_dispose_view(data->gl_view);
     destroy_cocoa_window(data);
 
     CFDictionaryRemoveValue(win_datas, hwnd);
@@ -853,11 +855,11 @@ void CDECL macdrv_SetFocus(HWND hwnd)
     if (!(hwnd = GetAncestor(hwnd, GA_ROOT))) return;
     if (!(data = get_win_data(hwnd))) return;
 
-    if (data->cocoa_window)
+    if (data->cocoa_window && data->on_screen)
     {
+        BOOL activate = activate_on_focus_time && (GetTickCount() - activate_on_focus_time < 2000);
         /* Set Mac focus */
-        macdrv_give_cocoa_window_focus(data->cocoa_window);
-        data->on_screen = TRUE;
+        macdrv_give_cocoa_window_focus(data->cocoa_window, activate);
     }
 
     release_win_data(data);
@@ -915,6 +917,8 @@ void CDECL macdrv_SetParent(HWND hwnd, HWND parent, HWND old_parent)
     else  /* new top level window */
         create_cocoa_window(data);
     release_win_data(data);
+
+    set_gl_view_parent(hwnd, parent);
 }
 
 
@@ -970,6 +974,7 @@ void CDECL macdrv_SetWindowStyle(HWND hwnd, INT offset, STYLESTRUCT *style)
         if (offset == GWL_EXSTYLE && (changed & WS_EX_LAYERED)) /* changing WS_EX_LAYERED resets attributes */
         {
             data->layered = FALSE;
+            data->ulw_layered = FALSE;
             sync_window_opacity(data, 0, 0, FALSE, 0);
             if (data->surface) set_surface_use_alpha(data->surface, FALSE);
         }
@@ -988,7 +993,7 @@ void CDECL macdrv_SetWindowText(HWND hwnd, LPCWSTR text)
 
     TRACE("%p, %s\n", hwnd, debugstr_w(text));
 
-    if ((win = macdrv_get_cocoa_window(hwnd)))
+    if ((win = macdrv_get_cocoa_window(hwnd, FALSE)))
         macdrv_set_cocoa_window_title(win, text, strlenW(text));
 }
 
@@ -1091,6 +1096,7 @@ BOOL CDECL macdrv_UpdateLayeredWindow(HWND hwnd, const UPDATELAYEREDWINDOWINFO *
     if (!(data = get_win_data(hwnd))) return FALSE;
 
     data->layered = TRUE;
+    data->ulw_layered = TRUE;
 
     rect = *window_rect;
     OffsetRect(&rect, -window_rect->left, -window_rect->top);
@@ -1098,7 +1104,7 @@ BOOL CDECL macdrv_UpdateLayeredWindow(HWND hwnd, const UPDATELAYEREDWINDOWINFO *
     surface = data->surface;
     if (!surface || memcmp(&surface->rect, &rect, sizeof(RECT)))
     {
-        data->surface = create_surface(data->cocoa_window, &rect, TRUE);
+        data->surface = create_surface(data->cocoa_window, &rect, NULL, TRUE);
         set_window_surface(data->cocoa_window, data->surface);
         if (surface) window_surface_release(surface);
         surface = data->surface;
@@ -1190,6 +1196,44 @@ LRESULT CDECL macdrv_WindowMessage(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
             release_win_data(data);
         }
         return 0;
+    case WM_MACDRV_UPDATE_DESKTOP_RECT:
+        if (hwnd == GetDesktopWindow())
+        {
+            CGRect new_desktop_rect;
+            RECT current_desktop_rect;
+
+            macdrv_reset_device_metrics();
+            new_desktop_rect = macdrv_get_desktop_rect();
+            if (!GetWindowRect(hwnd, &current_desktop_rect) ||
+                !CGRectEqualToRect(cgrect_from_rect(current_desktop_rect), new_desktop_rect))
+            {
+                SendMessageTimeoutW(HWND_BROADCAST, WM_MACDRV_RESET_DEVICE_METRICS, 0, 0,
+                                    SMTO_ABORTIFHUNG, 2000, NULL);
+                SetWindowPos(hwnd, 0, CGRectGetMinX(new_desktop_rect), CGRectGetMinY(new_desktop_rect),
+                             CGRectGetWidth(new_desktop_rect), CGRectGetHeight(new_desktop_rect),
+                             SWP_NOZORDER | SWP_NOACTIVATE | SWP_DEFERERASE);
+                SendMessageTimeoutW(HWND_BROADCAST, WM_MACDRV_DISPLAYCHANGE, wp, lp,
+                                    SMTO_ABORTIFHUNG, 2000, NULL);
+            }
+        }
+        return 0;
+    case WM_MACDRV_RESET_DEVICE_METRICS:
+        macdrv_reset_device_metrics();
+        return 0;
+    case WM_MACDRV_DISPLAYCHANGE:
+        if ((data = get_win_data(hwnd)))
+        {
+            if (data->cocoa_window && data->on_screen)
+                sync_window_position(data, SWP_NOZORDER | SWP_NOACTIVATE);
+            release_win_data(data);
+        }
+        SendMessageW(hwnd, WM_DISPLAYCHANGE, wp, lp);
+        return 0;
+    case WM_MACDRV_ACTIVATE_ON_FOLLOWING_FOCUS:
+        activate_on_focus_time = GetTickCount();
+        if (!activate_on_focus_time) activate_on_focus_time = 1;
+        TRACE("WM_MACDRV_ACTIVATE_ON_FOLLOWING_FOCUS time %u\n", activate_on_focus_time);
+        break;
     }
 
     FIXME("unrecognized window msg %x hwnd %p wp %lx lp %lx\n", msg, hwnd, wp, lp);
@@ -1237,6 +1281,7 @@ void CDECL macdrv_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags
     /* create the window surface if necessary */
     if (!data->cocoa_window) goto done;
     if (swp_flags & SWP_HIDEWINDOW) goto done;
+    if (data->ulw_layered) goto done;
 
     if (*surface) window_surface_release(*surface);
     *surface = NULL;
@@ -1247,6 +1292,7 @@ void CDECL macdrv_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags
         if (!memcmp(&data->surface->rect, &surface_rect, sizeof(surface_rect)))
         {
             /* existing surface is good enough */
+            surface_clip_to_visible_rect(data->surface, visible_rect);
             window_surface_add_ref(data->surface);
             *surface = data->surface;
             goto done;
@@ -1254,7 +1300,7 @@ void CDECL macdrv_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags
     }
     else if (!(swp_flags & SWP_SHOWWINDOW) && !(style & WS_VISIBLE)) goto done;
 
-    *surface = create_surface(data->cocoa_window, &surface_rect, FALSE);
+    *surface = create_surface(data->cocoa_window, &surface_rect, data->surface, FALSE);
 
 done:
     release_win_data(data);
@@ -1284,11 +1330,13 @@ void CDECL macdrv_WindowPosChanged(HWND hwnd, HWND insert_after, UINT swp_flags,
     data->window_rect = *window_rect;
     data->whole_rect  = *visible_rect;
     data->client_rect = *client_rect;
-    if (surface)
-        window_surface_add_ref(surface);
-    set_window_surface(data->cocoa_window, surface);
-    if (data->surface) window_surface_release(data->surface);
-    data->surface = surface;
+    if (!data->ulw_layered)
+    {
+        if (surface) window_surface_add_ref(surface);
+        set_window_surface(data->cocoa_window, surface);
+        if (data->surface) window_surface_release(data->surface);
+        data->surface = surface;
+    }
 
     TRACE("win %p/%p window %s whole %s client %s style %08x flags %08x surface %p\n",
            hwnd, data->cocoa_window, wine_dbgstr_rect(window_rect),
@@ -1329,6 +1377,8 @@ void CDECL macdrv_WindowPosChanged(HWND hwnd, HWND insert_after, UINT swp_flags,
         }
     }
 
+    sync_gl_view(data);
+
     if (!data->cocoa_window) goto done;
 
     if (data->on_screen)
@@ -1337,17 +1387,6 @@ void CDECL macdrv_WindowPosChanged(HWND hwnd, HWND insert_after, UINT swp_flags,
             hide_window(data);
     }
 
-    if (new_style & WS_VISIBLE)
-    {
-        if (!data->on_screen || (swp_flags & (SWP_FRAMECHANGED|SWP_STATECHANGED)))
-            set_cocoa_window_properties(data);
-
-        /* layered windows are not shown until their attributes are set */
-        if (!data->on_screen &&
-            (data->layered || !(GetWindowLongW( hwnd, GWL_EXSTYLE ) & WS_EX_LAYERED)))
-            show_window(data);
-    }
-
     /* check if we are currently processing an event relevant to this window */
     if (!thread_data || !thread_data->current_event ||
         thread_data->current_event->window != data->cocoa_window ||
@@ -1359,6 +1398,17 @@ void CDECL macdrv_WindowPosChanged(HWND hwnd, HWND insert_after, UINT swp_flags,
         set_cocoa_window_properties(data);
     }
 
+    if (new_style & WS_VISIBLE)
+    {
+        if (!data->on_screen || (swp_flags & (SWP_FRAMECHANGED|SWP_STATECHANGED)))
+            set_cocoa_window_properties(data);
+
+        /* layered windows are not shown until their attributes are set */
+        if (!data->on_screen &&
+            (data->layered || !(GetWindowLongW( hwnd, GWL_EXSTYLE ) & WS_EX_LAYERED)))
+            show_window(data);
+    }
+
 done:
     release_win_data(data);
 }
@@ -1596,3 +1646,170 @@ void macdrv_window_did_unminimize(HWND hwnd)
 done:
     release_win_data(data);
 }
+
+
+struct quit_info {
+    HWND               *wins;
+    UINT                capacity;
+    UINT                count;
+    UINT                done;
+    DWORD               flags;
+    BOOL                result;
+    BOOL                replied;
+};
+
+
+static BOOL CALLBACK get_process_windows(HWND hwnd, LPARAM lp)
+{
+    struct quit_info *qi = (struct quit_info*)lp;
+    DWORD pid;
+
+    GetWindowThreadProcessId(hwnd, &pid);
+    if (pid == GetCurrentProcessId())
+    {
+        if (qi->count >= qi->capacity)
+        {
+            UINT new_cap = qi->capacity * 2;
+            HWND *new_wins = HeapReAlloc(GetProcessHeap(), 0, qi->wins,
+                                         new_cap * sizeof(*qi->wins));
+            if (!new_wins) return FALSE;
+            qi->wins = new_wins;
+            qi->capacity = new_cap;
+        }
+
+        qi->wins[qi->count++] = hwnd;
+    }
+
+    return TRUE;
+}
+
+
+static void CALLBACK quit_callback(HWND hwnd, UINT msg, ULONG_PTR data, LRESULT result)
+{
+    struct quit_info *qi = (struct quit_info*)data;
+
+    qi->done++;
+
+    if (msg == WM_QUERYENDSESSION)
+    {
+        TRACE("got WM_QUERYENDSESSION result %ld from win %p (%u of %u done)\n", result,
+              hwnd, qi->done, qi->count);
+
+        if (!result && qi->result)
+        {
+            qi->result = FALSE;
+
+            /* On the first FALSE from WM_QUERYENDSESSION, we already know the
+               ultimate reply.  Might as well tell Cocoa now. */
+            if (!qi->replied)
+            {
+                qi->replied = TRUE;
+                TRACE("giving quit reply %d\n", qi->result);
+                macdrv_quit_reply(qi->result);
+            }
+        }
+
+        if (qi->done >= qi->count)
+        {
+            UINT i;
+
+            qi->done = 0;
+            for (i = 0; i < qi->count; i++)
+            {
+                TRACE("sending WM_ENDSESSION to win %p result %d flags 0x%08x\n", qi->wins[i],
+                      qi->result, qi->flags);
+                if (!SendMessageCallbackW(qi->wins[i], WM_ENDSESSION, qi->result, qi->flags,
+                                          quit_callback, (ULONG_PTR)qi))
+                {
+                    WARN("failed to send WM_ENDSESSION to win %p; error 0x%08x\n",
+                         qi->wins[i], GetLastError());
+                    quit_callback(qi->wins[i], WM_ENDSESSION, (ULONG_PTR)qi, 0);
+                }
+            }
+        }
+    }
+    else /* WM_ENDSESSION */
+    {
+        TRACE("finished WM_ENDSESSION for win %p (%u of %u done)\n", hwnd, qi->done, qi->count);
+
+        if (qi->done >= qi->count)
+        {
+            if (!qi->replied)
+            {
+                TRACE("giving quit reply %d\n", qi->result);
+                macdrv_quit_reply(qi->result);
+            }
+
+            TRACE("%sterminating process\n", qi->result ? "" : "not ");
+            if (qi->result)
+                TerminateProcess(GetCurrentProcess(), 0);
+
+            HeapFree(GetProcessHeap(), 0, qi->wins);
+            HeapFree(GetProcessHeap(), 0, qi);
+        }
+    }
+}
+
+
+/***********************************************************************
+ *              macdrv_app_quit_requested
+ *
+ * Handler for APP_QUIT_REQUESTED events.
+ */
+void macdrv_app_quit_requested(const macdrv_event *event)
+{
+    struct quit_info *qi;
+    UINT i;
+
+    TRACE("reason %d\n", event->app_quit_requested.reason);
+
+    qi = HeapAlloc(GetProcessHeap(), 0, sizeof(*qi));
+    if (!qi)
+        goto fail;
+
+    qi->capacity = 32;
+    qi->wins = HeapAlloc(GetProcessHeap(), 0, qi->capacity * sizeof(*qi->wins));
+    qi->count = qi->done = 0;
+
+    if (!qi->wins || !EnumWindows(get_process_windows, (LPARAM)qi))
+        goto fail;
+
+    switch (event->app_quit_requested.reason)
+    {
+        case QUIT_REASON_LOGOUT:
+        default:
+            qi->flags = ENDSESSION_LOGOFF;
+            break;
+        case QUIT_REASON_RESTART:
+        case QUIT_REASON_SHUTDOWN:
+            qi->flags = 0;
+            break;
+    }
+
+    qi->result = TRUE;
+    qi->replied = FALSE;
+
+    for (i = 0; i < qi->count; i++)
+    {
+        TRACE("sending WM_QUERYENDSESSION to win %p\n", qi->wins[i]);
+        if (!SendMessageCallbackW(qi->wins[i], WM_QUERYENDSESSION, 0, qi->flags,
+                                  quit_callback, (ULONG_PTR)qi))
+        {
+            WARN("failed to send WM_QUERYENDSESSION to win %p; error 0x%08x; assuming refusal\n",
+                 qi->wins[i], GetLastError());
+            quit_callback(qi->wins[i], WM_QUERYENDSESSION, (ULONG_PTR)qi, FALSE);
+        }
+    }
+
+    /* quit_callback() will clean up qi */
+    return;
+
+fail:
+    WARN("failed to allocate window list\n");
+    if (qi)
+    {
+        HeapFree(GetProcessHeap(), 0, qi->wins);
+        HeapFree(GetProcessHeap(), 0, qi);
+    }
+    macdrv_quit_reply(FALSE);
+}
