@@ -353,8 +353,8 @@ extern int mkstemps(char *template, int suffix_len);
 # define _P_NOWAITO 3
 # define _P_DETACH  4
 #endif
-#ifndef HAVE_SPAWNVP
-extern int spawnvp(int mode, const char *cmdname, const char * const argv[]);
+#ifndef HAVE__SPAWNVP
+extern int _spawnvp(int mode, const char *cmdname, const char * const argv[]);
 #endif
 
 /* Interlocked functions */
