@@ -213,6 +213,7 @@ struct cpu_register
 #define REG(r,gs)  {FIELD_OFFSET(CONTEXT, r), sizeof(((CONTEXT*)NULL)->r), gs}
 
 #ifdef __i386__
+static const char target_xml[] = "";
 static struct cpu_register cpu_register_map[] = {
     REG(Eax, 4),
     REG(Ecx, 4),
@@ -232,6 +233,7 @@ static struct cpu_register cpu_register_map[] = {
     REG(SegGs, 4),
 };
 #elif defined(__powerpc__)
+static const char target_xml[] = "";
 static struct cpu_register cpu_register_map[] = {
     REG(Gpr0, 4),
     REG(Gpr1, 4),
@@ -308,6 +310,7 @@ static struct cpu_register cpu_register_map[] = {
     /* see gdb/nlm/ppc.c */
 };
 #elif defined(__x86_64__)
+static const char target_xml[] = "";
 static struct cpu_register cpu_register_map[] = {
     REG(Rax, 8),
     REG(Rbx, 8),
@@ -334,42 +337,29 @@ static struct cpu_register cpu_register_map[] = {
     REG(SegFs, 4),
     REG(SegGs, 4),
 };
-#elif defined(__sparc__)
-static struct cpu_register cpu_register_map[] = {
-    REG(g0, 4),
-    REG(g1, 4),
-    REG(g2, 4),
-    REG(g3, 4),
-    REG(g4, 4),
-    REG(g5, 4),
-    REG(g6, 4),
-    REG(g7, 4),
-    REG(o0, 4),
-    REG(o1, 4),
-    REG(o2, 4),
-    REG(o3, 4),
-    REG(o4, 4),
-    REG(o5, 4),
-    REG(o6, 4),
-    REG(o7, 4),
-    REG(l0, 4),
-    REG(l1, 4),
-    REG(l2, 4),
-    REG(l3, 4),
-    REG(l4, 4),
-    REG(l5, 4),
-    REG(l6, 4),
-    REG(l7, 4),
-    REG(i0, 4),
-    REG(i1, 4),
-    REG(i2, 4),
-    REG(i3, 4),
-    REG(i4, 4),
-    REG(i5, 4),
-    REG(i6, 4),
-    REG(i7, 4),
-};
 #elif defined(__arm__)
+static const char target_xml[] =
+    "l <target><architecture>arm</architecture>\n"
+    "<feature name=\"org.gnu.gdb.arm.core\">\n"
+    "    <reg name=\"r0\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r1\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r2\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r3\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r4\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r5\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r6\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r7\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r8\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r9\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r10\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r11\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"r12\" bitsize=\"32\" type=\"uint32\"/>\n"
+    "    <reg name=\"sp\" bitsize=\"32\" type=\"data_ptr\"/>\n"
+    "    <reg name=\"lr\" bitsize=\"32\"/>\n"
+    "    <reg name=\"pc\" bitsize=\"32\" type=\"code_ptr\"/>\n"
+    "    <reg name=\"cpsr\" bitsize=\"32\"/>\n"
+    "</feature></target>\n";
+
 static struct cpu_register cpu_register_map[] = {
     REG(R0, 4),
     REG(R1, 4),
@@ -387,8 +377,10 @@ static struct cpu_register cpu_register_map[] = {
     REG(Sp, 4),
     REG(Lr, 4),
     REG(Pc, 4),
+    REG(Cpsr, 4),
 };
 #elif defined(__aarch64__)
+static const char target_xml[] = "";
 static struct cpu_register cpu_register_map[] = {
     REG(X0,  8),
     REG(X1,  8),
@@ -448,6 +440,7 @@ static inline DWORD64   cpu_register(CONTEXT* ctx, unsigned idx)
     default:
         fprintf(stderr, "got unexpected size: %u\n", (unsigned)cpu_register_map[idx].ctx_length);
         assert(0);
+        return 0;
     }
 }
 
@@ -1930,10 +1923,15 @@ static enum packet_return packet_query(struct gdb_context* gdbctx)
             return packet_ok;
         if (strncmp(gdbctx->in_packet, "Supported", 9) == 0)
         {
-            /* no features supported */
-            packet_reply_open(gdbctx);
-            packet_reply_close(gdbctx);
-            return packet_done;
+            if (strlen(target_xml))
+                return packet_reply(gdbctx, "PacketSize=400;qXfer:features:read+", -1);
+            else
+            {
+                /* no features supported */
+                packet_reply_open(gdbctx);
+                packet_reply_close(gdbctx);
+                return packet_done;
+            }
         }
         break;
     case 'T':
@@ -1961,6 +1959,10 @@ static enum packet_return packet_query(struct gdb_context* gdbctx)
             return packet_done;
         }
         break;
+    case 'X':
+        if (strlen(target_xml) && strncmp(gdbctx->in_packet, "Xfer:features:read:target.xml", 29) == 0)
+            return packet_reply(gdbctx, target_xml, -1);
+        break;
     }
     if (gdbctx->trace & GDBPXY_TRC_COMMAND_ERROR)
         fprintf(stderr, "Unknown or malformed query %*.*s\n",
@@ -2326,7 +2328,7 @@ static BOOL gdb_startup(struct gdb_context* gdbctx, DEBUG_EVENT* de, unsigned fl
 {
     int                 sock;
     struct sockaddr_in  s_addrs;
-    unsigned int        s_len = sizeof(s_addrs);
+    socklen_t           s_len = sizeof(s_addrs);
     struct pollfd       pollfd;
     IMAGEHLP_MODULE64   imh_mod;
     BOOL                ret = FALSE;
