@@ -20,4 +20,16 @@
 
 extern IHost host_obj;
 
+extern IArguments2 arguments_obj;
+
 extern ITypeInfo *host_ti;
+
+extern ITypeInfo *arguments_ti;
+
+extern WCHAR scriptFullName[];
+
+extern WCHAR **argums;
+
+extern int numOfArgs;
+
+extern VARIANT_BOOL wshInteractive;
