@@ -947,7 +947,7 @@ DECL_HANDLER(new_process)
 
     if (!(thread = create_process( socket_fd, current, req->inherit_all ))) goto done;
     process = thread->process;
-    process->debug_children = !(req->create_flags & DEBUG_ONLY_THIS_PROCESS);
+    process->debug_children = !!(req->create_flags & DEBUG_PROCESS);
     process->startup_info = (struct startup_info *)grab_object( info );
 
     /* connect to the window station */
@@ -1082,12 +1082,16 @@ DECL_HANDLER(terminate_process)
 {
     struct process *process;
 
-    if ((process = get_process_from_handle( req->handle, PROCESS_TERMINATE )))
+    if (req->handle)
     {
-        reply->self = (current->process == process);
-        terminate_process( process, current, req->exit_code );
-        release_object( process );
+        process = get_process_from_handle( req->handle, PROCESS_TERMINATE );
+        if (!process) return;
     }
+    else process = (struct process *)grab_object( current->process );
+
+    reply->self = (current->process == process);
+    terminate_process( process, current, req->exit_code );
+    release_object( process );
 }
 
 /* fetch information about a process */
