@@ -43,6 +43,7 @@
 #include "file.h"
 #include "handle.h"
 #include "request.h"
+#include "process.h"
 #include "unicode.h"
 #include "security.h"
 
@@ -100,8 +101,8 @@ struct key_value
 #define MIN_SUBKEYS  8   /* min. number of allocated subkeys per key */
 #define MIN_VALUES   8   /* min. number of allocated values per key */
 
-#define MAX_NAME_LEN  MAX_PATH  /* max. length of a key name */
-#define MAX_VALUE_LEN MAX_PATH  /* max. length of a value name */
+#define MAX_NAME_LEN  255    /* max. length of a key name */
+#define MAX_VALUE_LEN 16383  /* max. length of a value name */
 
 /* the root of the registry tree */
 static struct key *root_key;
@@ -109,6 +110,7 @@ static struct key *root_key;
 static const timeout_t ticks_1601_to_1970 = (timeout_t)86400 * (369 * 365 + 89) * TICKS_PER_SEC;
 static const timeout_t save_period = 30 * -TICKS_PER_SEC;  /* delay between periodic saves */
 static struct timeout_user *save_timeout_user;  /* saving timer */
+static enum prefix_type { PREFIX_UNKNOWN, PREFIX_32BIT, PREFIX_64BIT } prefix_type;
 
 static const WCHAR root_name[] = { '\\','R','e','g','i','s','t','r','y','\\' };
 static const WCHAR wow6432node[] = {'W','o','w','6','4','3','2','N','o','d','e'};
@@ -145,6 +147,7 @@ struct file_load_info
 
 static void key_dump( struct object *obj, int verbose );
 static unsigned int key_map_access( struct object *obj, unsigned int access );
+static struct security_descriptor *key_get_sd( struct object *obj );
 static int key_close_handle( struct object *obj, struct process *process, obj_handle_t handle );
 static void key_destroy( struct object *obj );
 
@@ -160,7 +163,7 @@ static const struct object_ops key_ops =
     no_signal,               /* signal */
     no_get_fd,               /* get_fd */
     key_map_access,          /* map_access */
-    default_get_sd,          /* get_sd */
+    key_get_sd,              /* get_sd */
     default_set_sd,          /* set_sd */
     no_lookup_name,          /* lookup_name */
     no_open_file,            /* open_file */
@@ -329,7 +332,48 @@ static unsigned int key_map_access( struct object *obj, unsigned int access )
     if (access & GENERIC_WRITE)   access |= KEY_WRITE;
     if (access & GENERIC_EXECUTE) access |= KEY_EXECUTE;
     if (access & GENERIC_ALL)     access |= KEY_ALL_ACCESS;
-    return access & ~(GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | GENERIC_ALL);
+    /* filter the WOW64 masks, as they aren't real access bits */
+    return access & ~(GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | GENERIC_ALL |
+                      KEY_WOW64_64KEY | KEY_WOW64_32KEY);
+}
+
+static struct security_descriptor *key_get_sd( struct object *obj )
+{
+    static struct security_descriptor *key_default_sd;
+
+    if (obj->sd) return obj->sd;
+
+    if (!key_default_sd)
+    {
+        size_t users_sid_len = security_sid_len( security_builtin_users_sid );
+        size_t admins_sid_len = security_sid_len( security_builtin_admins_sid );
+        size_t dacl_len = sizeof(ACL) + offsetof( ACCESS_ALLOWED_ACE, SidStart ) + users_sid_len;
+        ACCESS_ALLOWED_ACE *aaa;
+        ACL *dacl;
+
+        key_default_sd = mem_alloc( sizeof(*key_default_sd) + 2 * admins_sid_len + dacl_len );
+        key_default_sd->control   = SE_DACL_PRESENT;
+        key_default_sd->owner_len = admins_sid_len;
+        key_default_sd->group_len = admins_sid_len;
+        key_default_sd->sacl_len  = 0;
+        key_default_sd->dacl_len  = dacl_len;
+        memcpy( key_default_sd + 1, security_builtin_admins_sid, admins_sid_len );
+        memcpy( (char *)(key_default_sd + 1) + admins_sid_len, security_builtin_admins_sid, admins_sid_len );
+
+        dacl = (ACL *)((char *)(key_default_sd + 1) + 2 * admins_sid_len);
+        dacl->AclRevision = ACL_REVISION;
+        dacl->Sbz1 = 0;
+        dacl->AclSize = dacl_len;
+        dacl->AceCount = 1;
+        dacl->Sbz2 = 0;
+        aaa = (ACCESS_ALLOWED_ACE *)(dacl + 1);
+        aaa->Header.AceType = ACCESS_ALLOWED_ACE_TYPE;
+        aaa->Header.AceFlags = INHERIT_ONLY_ACE | CONTAINER_INHERIT_ACE;
+        aaa->Header.AceSize = offsetof( ACCESS_ALLOWED_ACE, SidStart ) + users_sid_len;
+        aaa->Mask = GENERIC_READ;
+        memcpy( &aaa->SidStart, security_builtin_users_sid, users_sid_len );
+    }
+    return key_default_sd;
 }
 
 /* close the notification associated with a handle */
@@ -537,7 +581,8 @@ static struct key *alloc_subkey( struct key *parent, const struct unicode_str *n
         for (i = ++parent->last_subkey; i > index; i--)
             parent->subkeys[i] = parent->subkeys[i-1];
         parent->subkeys[index] = key;
-        if (is_wow6432node( key->name, key->namelen )) parent->flags |= KEY_WOW64;
+        if (is_wow6432node( key->name, key->namelen ) && !is_wow6432node( parent->name, parent->namelen ))
+            parent->flags |= KEY_WOW64;
     }
     return key;
 }
@@ -711,12 +756,6 @@ static struct key *create_key( struct key *key, const struct unicode_str *name,
     int index;
     struct unicode_str token, next;
 
-    if (key->flags & KEY_DELETED) /* we cannot create a subkey under a deleted key */
-    {
-        set_error( STATUS_KEY_DELETED );
-        return NULL;
-    }
-
     *created = 0;
     if (!(key = open_key_prefix( key, name, access, &token, &index ))) return NULL;
 
@@ -901,7 +940,7 @@ static void enum_key( const struct key *key, int index, int info_class,
 static int delete_key( struct key *key, int recurse )
 {
     int index;
-    struct key *parent;
+    struct key *parent = key->parent;
 
     /* must find parent and index */
     if (key == root_key)
@@ -909,11 +948,7 @@ static int delete_key( struct key *key, int recurse )
         set_error( STATUS_ACCESS_DENIED );
         return -1;
     }
-    if (!(parent = key->parent) || (key->flags & KEY_DELETED))
-    {
-        set_error( STATUS_KEY_DELETED );
-        return -1;
-    }
+    assert( parent );
 
     while (recurse && (key->last_subkey>=0))
         if (0 > delete_key(key->subkeys[key->last_subkey], 1))
@@ -1164,16 +1199,24 @@ static void delete_value( struct key *key, const struct unicode_str *name )
 }
 
 /* get the registry key corresponding to an hkey handle */
-static inline struct key *get_hkey_obj( obj_handle_t hkey, unsigned int access )
+static struct key *get_hkey_obj( obj_handle_t hkey, unsigned int access )
 {
-    return (struct key *)get_handle_obj( current->process, hkey, access, &key_ops );
+    struct key *key = (struct key *)get_handle_obj( current->process, hkey, access, &key_ops );
+
+    if (key && key->flags & KEY_DELETED)
+    {
+        set_error( STATUS_KEY_DELETED );
+        release_object( key );
+        key = NULL;
+    }
+    return key;
 }
 
 /* get the registry key corresponding to a parent key handle */
 static inline struct key *get_parent_hkey_obj( obj_handle_t hkey )
 {
     if (!hkey) return (struct key *)grab_object( root_key );
-    return (struct key *)get_handle_obj( current->process, hkey, 0, &key_ops );
+    return get_hkey_obj( hkey, 0 );
 }
 
 /* read a line from the input file */
@@ -1306,6 +1349,35 @@ static struct key *load_key( struct key *base, const char *buffer,
     return create_key_recursive( base, &name, modif );
 }
 
+/* load a global option from the input file */
+static int load_global_option( const char *buffer, struct file_load_info *info )
+{
+    const char *p;
+
+    if (!strncmp( buffer, "#arch=", 6 ))
+    {
+        enum prefix_type type;
+        p = buffer + 6;
+        if (!strcmp( p, "win32" )) type = PREFIX_32BIT;
+        else if (!strcmp( p, "win64" )) type = PREFIX_64BIT;
+        else
+        {
+            file_read_error( "Unknown architecture", info );
+            set_error( STATUS_NOT_REGISTRY_FILE );
+            return 0;
+        }
+        if (prefix_type == PREFIX_UNKNOWN) prefix_type = type;
+        else if (type != prefix_type)
+        {
+            file_read_error( "Mismatched architecture", info );
+            set_error( STATUS_NOT_REGISTRY_FILE );
+            return 0;
+        }
+    }
+    /* ignore unknown options */
+    return 1;
+}
+
 /* load a key option from the input file */
 static int load_key_option( struct key *key, const char *buffer, struct file_load_info *info )
 {
@@ -1526,6 +1598,7 @@ static void load_keys( struct key *key, const char *filename, FILE *f, int prefi
             break;
         case '#':   /* option */
             if (subkey) load_key_option( subkey, p, &info );
+            else if (!load_global_option( p, &info )) goto done;
             break;
         case ';':   /* comment */
         case 0:     /* empty line */
@@ -1564,7 +1637,7 @@ static void load_registry( struct key *key, obj_handle_t handle )
 }
 
 /* load one of the initial registry files */
-static void load_init_registry_from_file( const char *filename, struct key *key )
+static int load_init_registry_from_file( const char *filename, struct key *key )
 {
     FILE *f;
 
@@ -1575,7 +1648,7 @@ static void load_init_registry_from_file( const char *filename, struct key *key
         if (get_error() == STATUS_NOT_REGISTRY_FILE)
         {
             fprintf( stderr, "%s is not a valid registry file\n", filename );
-            return;
+            return 1;
         }
     }
 
@@ -1584,6 +1657,7 @@ static void load_init_registry_from_file( const char *filename, struct key *key
     save_branch_info[save_branch_count].path = filename;
     save_branch_info[save_branch_count++].key = (struct key *)grab_object( key );
     make_object_static( &key->obj );
+    return (f != NULL);
 }
 
 static WCHAR *format_user_registry_path( const SID *sid, struct unicode_str *path )
@@ -1609,6 +1683,34 @@ static WCHAR *format_user_registry_path( const SID *sid, struct unicode_str *pat
     return p;
 }
 
+/* get the cpu architectures that can be supported in the current prefix */
+unsigned int get_prefix_cpu_mask(void)
+{
+    /* Allowed server/client/prefix combinations:
+     *
+     *              prefix
+     *            32     64
+     *  server +------+------+ client
+     *         |  ok  | fail | 32
+     *      32 +------+------+---
+     *         | fail | fail | 64
+     *      ---+------+------+---
+     *         |  ok  |  ok  | 32
+     *      64 +------+------+---
+     *         | fail |  ok  | 64
+     *      ---+------+------+---
+     */
+    switch (prefix_type)
+    {
+    case PREFIX_64BIT:
+        /* 64-bit prefix requires 64-bit server */
+        return sizeof(void *) > sizeof(int) ? ~0 : 0;
+    case PREFIX_32BIT:
+    default:
+        return ~CPU_64BIT_MASK;  /* only 32-bit cpus supported on 32-bit prefix */
+    }
+}
+
 /* registry initialisation */
 void init_registry(void)
 {
@@ -1640,7 +1742,10 @@ void init_registry(void)
     if (!(hklm = create_key_recursive( root_key, &HKLM_name, current_time )))
         fatal_error( "could not create Machine registry key\n" );
 
-    load_init_registry_from_file( "system.reg", hklm );
+    if (!load_init_registry_from_file( "system.reg", hklm ))
+        prefix_type = sizeof(void *) > sizeof(int) ? PREFIX_64BIT : PREFIX_32BIT;
+    else if (prefix_type == PREFIX_UNKNOWN)
+        prefix_type = PREFIX_32BIT;
 
     /* load userdef.reg into Registry\User\.Default */
 
@@ -1653,7 +1758,7 @@ void init_registry(void)
     /* load user.reg into HKEY_CURRENT_USER */
 
     /* FIXME: match default user in token.c. should get from process token instead */
-    current_user_path = format_user_registry_path( security_interactive_sid, &current_user_str );
+    current_user_path = format_user_registry_path( security_local_user_sid, &current_user_str );
     if (!current_user_path ||
         !(hkcu = create_key_recursive( root_key, &current_user_str, current_time )))
         fatal_error( "could not create HKEY_CURRENT_USER registry key\n" );
@@ -1661,7 +1766,7 @@ void init_registry(void)
     load_init_registry_from_file( "user.reg", hkcu );
 
     /* set the shared flag on Software\Classes\Wow6432Node */
-    if (sizeof(void *) > sizeof(int))
+    if (prefix_type == PREFIX_64BIT)
     {
         if ((key = create_key_recursive( hklm, &classes_name, current_time )))
         {
@@ -1688,6 +1793,17 @@ static void save_all_subkeys( struct key *key, FILE *f )
     fprintf( f, ";; All keys relative to " );
     dump_path( key, NULL, f );
     fprintf( f, "\n" );
+    switch (prefix_type)
+    {
+    case PREFIX_32BIT:
+        fprintf( f, "\n#arch=win32\n" );
+        break;
+    case PREFIX_64BIT:
+        fprintf( f, "\n#arch=win64\n" );
+        break;
+    default:
+        break;
+    }
     save_subkeys( key, key, f );
 }
 
@@ -1697,11 +1813,6 @@ static void save_registry( struct key *key, obj_handle_t handle )
     struct file *file;
     int fd;
 
-    if (key->flags & KEY_DELETED)
-    {
-        set_error( STATUS_KEY_DELETED );
-        return;
-    }
     if (!(file = get_file_obj( current->process, handle, FILE_WRITE_DATA ))) return;
     fd = dup( get_file_unix_fd( file ) );
     release_object( file );
@@ -1833,6 +1944,12 @@ void flush_registry(void)
     if (fchdir( server_dir_fd ) == -1) fatal_perror( "chdir to server dir" );
 }
 
+/* determine if the thread is wow64 (32-bit client running on 64-bit prefix) */
+static int is_wow64_thread( struct thread *thread )
+{
+    return (prefix_type == PREFIX_64BIT && !(CPU_FLAG(thread->process->cpu) & CPU_64BIT_MASK));
+}
+
 
 /* create a registry key */
 DECL_HANDLER(create_key)
