@@ -537,7 +537,7 @@ static void master_socket_poll_event( struct fd *fd, int event )
     else if (event & POLLIN)
     {
         struct sockaddr_un dummy;
-        unsigned int len = sizeof(dummy);
+        socklen_t len = sizeof(dummy);
         int client = accept( get_unix_fd( master_socket->fd ), (struct sockaddr *) &dummy, &len );
         if (client == -1) return;
         fcntl( client, F_SETFL, O_NONBLOCK );
